# User configuration
vmax <- read.csv('dcmip2016_25km_vmax.txt')
colnames(vmax) <- sub('.', '-', colnames(vmax), fixed=TRUE)
ps <- read.csv('dcmip2016_25km_ps.txt')
colnames(ps) <- sub('.', '-', colnames(ps), fixed=TRUE)
fit <- read.csv('dcmip2016_25km_fitparams.txt')
cols <- c('red', 'darkgreen', 'orange', 'maroon', 'cyan')
models <- toupper(colnames(vmax))
pt.cex <- 0.5
                 
# Plot setup
postscript('dcmip_windPres.ps', bg='white', paper='special',
    height=7, width=9, horiz=FALSE)
default <- par(mar=c(5,5,4,2))
plot(0, 0, ylim=c(10, 70), xlim=c(900, 1012), type='n', xaxs='i', yaxs='i',
     ylab=expression('Maximum 1000 m Wind (m s'^{-1}*')'),
     xlab='Central Pressure (hPa)',
     cex.axis=1.4, cex.lab=1.4, las=1)

# Plot DCMIP results
i <- 0
for (model in colnames(vmax)){
    i <- i+1
    points(ps[[model]], vmax[[model]], pch=19, col=cols[i], cex=pt.cex)
    fitfunc <- function(x){
        param <- fit[fit$model==model,]
        return(param$a * x^2 + param$b *x + param$c)
    }
    curve(fitfunc, lwd=1, col=cols[i], add=TRUE, 
          from=min(ps[[model]]), to=max(ps[[model]]))
}

# Plot new results
runs <- c('dgg25ctl', 'dgg25offb51')
runs.col <- c('blue', 'red')
runs.names <- c('GEM Control', 'OFFB51')
prefix <- '../vitals_f90/results/'
for (i in 1:length(runs)){
    run <- runs[i]
    ps.new <- c()
    vmax.new <- c()
    for (fcst in sprintf("%03d", seq(6, 240, by=6))){
        fname <- paste0(prefix,run,'/P0/azmean_',fcst,'.dat')
        ps.new <- c(ps.new, apply(read.table(fname), 1, min))
        wind <- read.table(paste0(prefix,run,'/VMLEV/azmean_',fcst,'.dat'))
        hght <- read.table(paste0(prefix,run,'/GZM/azmean_',fcst,'.dat'))[1:nrow(wind),]
        vmax.val <- 0
        for (rad in 1:ncol(hght)){
            vmax.val <- max(vmax.val, spline(hght[,rad], wind[,rad], xout=100.)$y)
        }
        vmax.new <- c(vmax.new, vmax.val)
    }
    points(ps.new, vmax.new, pch=21, bg=runs.col[i], cex=2*pt.cex, col='black')
}

legend('bottomleft', inset=0.01, legend=models, cex=1.4, col=cols,
       pch=19, pt.cex=0.5)
legend('topright', inset=0.01, legend=runs.names, cex=1.4, pt.bg=runs.col,
       pch=21, pt.cex=2*pt.cex, col='black')

par(default)
