fields <- c('wind', 'pres', 'track')
mod.names <- c( 'cmc', 'cmc4', 'ifs', 'shield', 'jma')
mod.long <- c('GEM Control', 'OFFB51', 'IFS47r1 (ECMWF)', 'SHiELD (GFDL)', 'JMA')
mod.cols <- c('blue', 'red', 'black', 'black', 'black')
mod.lwd <- c(6, 6, 3, 4, 4)
mod.lty <- c(1, 1, 1, 2, 4)
fcst.max <- 120
data.dir <- 'data/individual'

require(boot)
tcmean <- function(x, v){
    return(mean(x[v]))
}

gen.plot <- function(tc.mean, tc.ci, fld.name.long, err, range.y, legend.pos, units.y){

    mod.names.loc <- mod.names
    mod.long.loc <- mod.long
    mod.cols.loc <- mod.cols
    mod.lwd.loc <- mod.lwd
    mod.lty.loc <- mod.lty
    ref <- ''
    if (err == 'bias'){
        err.name.long <- 'Bias'
    } else if (err == 'rmse'){
        err.name.long <- 'Error'
    } else if (err == 'diff'){
        err.name.long <- 'Error Difference'
        i <- which(mod.names == 'ifs')
        mod.names.loc <- mod.names[-i]
        mod.long.loc <- mod.long[-i]
        mod.cols.loc <- mod.cols[-i]
        mod.lwd.loc <- mod.lwd[-i]
        mod.lty.loc <- mod.lty[-i]
        ref <- paste('Reference:',mod.long[i])
    }
        
    pdf(paste0('dimosic_',field,'_',err,'.pdf'), paper='special',
        height=7, width=9, bg='white')
    default <- par(mar=c(5,6,4,2))
    plot(1,1, type='n', xaxt='n', xlim=c(6, fcst.max), ylim=range.y, las=1, xaxs='i',
         cex.axis=1.6, cex.lab=1.6, cex.main=1.6, xlab='', ylab='',
         main=paste('DIMOSIC Global Tropical Cyclone', fld.name.long, err.name.long),
         panel.first={
             abline(h=0, col='grey', lty=2, lwd=3);
         })
    x.markup <- c(6, seq(24, fcst.max, by=24))
    axis(1, at=x.markup, cex.axis=1.6)
    axis(1, at=x.markup, labels=paste0("(",tc.n[fcsts %in% x.markup],")"),
         cex.axis=1.2, line=1., tick=FALSE)
    mtext('Forecast (h)', 1, cex=1.6, line=3.7)
    mtext(units.y, 2, cex=1.6, line=3.5)
    mtext('Count:', 1, cex=1.2, line=2., at=-10)
    polygon(c(fcsts, rev(fcsts)), c(tc.ci$cmc[,1], rev(tc.ci$cmc[,2])),
            col=adjustcolor('blue', alpha.f=0.4), border=NA)
    polygon(c(fcsts, rev(fcsts)), c(tc.ci$cmc4[,1], rev(tc.ci$cmc4[,2])),
            col=adjustcolor('red', alpha.f=0.4), border=NA)
    for (i in 1:length(mod.names.loc)){
        lines(fcsts, tc.mean[[mod.names.loc[i]]], lwd=mod.lwd.loc[i],
              col=mod.cols.loc[i], lty=mod.lty.loc[i])
    }
    legend(legend.pos, inset=0.02, legend=mod.long.loc, lty=mod.lty.loc, col=mod.cols.loc,
           lwd=mod.lwd.loc, cex=1.4, bg='white', seg.len=2.5)
    text(9, 55, ref, cex=1.4, adj=0)
    par(default)
    dev.off()
}

for (field in fields){
    
    if (field == 'track'){
        col.names <- c('fcst', 'n', 'ifs.rmse', 'shield.rmse', 'cmc.rmse', 'cmc4.rmse', 'jma.rmse')
    } else {
        col.names <- c('fcst', 'n', 'ifs.bias', 'ifs.rmse', 'shield.bias', 'shield.rmse',
                       'cmc.bias', 'cmc.rmse', 'cmc4.bias', 'cmc4.rmse', 'jma.bias', 'jma.rmse')
    }
    
    d.raw <- c()    
    for (file in list.files(data.dir, pattern=paste0('.*_',field,'_err.txt'))){
        d.raw <- rbind(d.raw, read.table(paste(data.dir, file, sep='/')))
    }
    
    colnames(d.raw) <- col.names
    fcsts <- as.numeric(levels(as.factor(d.raw$fcst)))
    fcsts <- fcsts[fcsts <= fcst.max]
    tc.n <- c()
    tc.bias.mean <- list()
    tc.bias.ci <- list()
    tc.rmse.mean <- list()
    tc.rmse.ci <- list()
    tc.rrmse.mean <- list()
    tc.rrmse.ci <- list()
    for (fcst in fcsts){        
        d.fcst <- d.raw[d.raw$fcst %in% fcst,]
        d <- c()
        for (i in 1:nrow(d.fcst)){
            n <- d.fcst$n[i]
            if (n > 0){
                for (j in 1:n){
                    d <- rbind(d, d.fcst[i,])
                }
            }
        }
        tc.n <- c(tc.n, nrow(d))
        for (mod in mod.names){
            tc.rmse.mean[[mod]] = c(tc.rmse.mean[[mod]], mean(d[[paste0(mod, '.rmse')]]))
            tc.rmse.ci[[mod]] = rbind(tc.rmse.ci[[mod]],
                                      boot.ci(boot(d[[paste0(mod, '.rmse')]], tcmean, R=1000),
                                              type='perc')$percent[4:5])
            if (field == 'track'){
                tc.rrmse.mean[[mod]] = c(tc.rrmse.mean[[mod]],
                                         mean(d[[paste0(mod, '.rmse')]]) - mean(d$ifs.rmse))
                tc.rrmse.ci[[mod]] = rbind(tc.rrmse.ci[[mod]],
                                           boot.ci(boot(d[[paste0(mod, '.rmse')]] - d$ifs.rmse, tcmean, R=1000),
                                                   type='perc')$percent[4:5])
            } else {
                tc.bias.mean[[mod]] = c(tc.bias.mean[[mod]], mean(d[[paste0(mod, '.bias')]]))
                tc.bias.ci[[mod]] = rbind(tc.bias.ci[[mod]],
                                          boot.ci(boot(d[[paste0(mod, '.bias')]], tcmean, R=1000),
                                                  type='perc')$percent[4:5])
            }
        }
    }

    if (field == 'wind'){
        units.y <- expression('Maximum Wind Speed (m s'^{-1}*')')
        gen.plot(tc.bias.mean, tc.bias.ci, 'Wind', 'bias', c(-25, -5), 'bottomleft', units.y)
        gen.plot(tc.rmse.mean, tc.rmse.ci, 'Wind', 'rmse', c(5, 20), 'bottom', units.y)
    } else if (field == 'pres'){
        units.y <- 'Central Pressure (hPa)'
        gen.plot(tc.bias.mean, tc.bias.ci, 'Pressure', 'bias', c(0, 20), 'bottomleft', units.y)
        gen.plot(tc.rmse.mean, tc.rmse.ci, 'Pressure', 'rmse', c(5, 20), 'bottom', units.y)
    } else if (field == 'track'){
        gen.plot(tc.rmse.mean, tc.rmse.ci, 'Track', 'rmse', c(0, 400), 'topleft', 'Track Error (km)')
        gen.plot(tc.rrmse.mean, tc.rrmse.ci, 'Track', 'diff', c(-25, 100), 'topleft',
                 'Track Error Difference (km)')
    }
    
}

