program process

  !  Associate required modules - MODIFY to include additional modules
  use Std_Variables, only: &
       ni,nj,nk,ntimes,t,progType,fileType,labl,dx,dy,dgrw,dim                  !variables
  use IO_Params, only: &
       input,output,gridFile,infoFld, &                                         !variables
       getParams                                                                !subprograms
  use Dimensions_Constructor, only: &
       dxs,dys, &                                                               !variables
       makeDim                                                                  !subprograms
  use TopLevel, only: &
       inOpen,outOpen,timer,fileTimer,projParam,lat,long,lev,vCoord,mapProj,gotLat,&!variables
       openGridFiles,closeGridFiles,getGridSize,getGridInfo,timeLoop            !subprograms
  use FileIO, only: &
       readGrid,writeGrid                                                       !subprograms
  use FileInfo_Type
  use Dimensions_Type
  use Math
  ! use Interface_MRG

  ! load FST       <- force comp90.d loading of ARMNLIB (FST library)
  ! load GEMPAK    <- force comp90.d loading of GEMLIB (GEMPAK library)
  ! load CDF       <- force comp90.d loading of NETCDF (netCDF library)

  !  This program template serves as the basis for the developer's
  !  interface for the MRG Interactive MetCal program and library.  
  !  Simple setup and IO operations are performed here; the developer
  !  is expected to extend upon this code as needed during further
  !  subprogram development for the library.

  implicit none

  !  Program Name: 
  !  Version: 1.0
  !  Programmer: 
  !  Date Modified:
  !  Modified by:

!!$    MRG_Interactive MetCal metorological calculator - developer's template
!!$    Copyright 2001 (C) Ron McTaggart-Cowan
!!$
!!$    This file is part of MRG Interactive MetCal (MIM).
!!$
!!$    MIM is free software; you can redistribute it and/or modify
!!$    it under the terms of the GNU General Public License as published by
!!$    the Free Software Foundation; either version 2 of the License or
!!$    any later version.
!!$
!!$    MIM is distributed in the hope that it will be useful,
!!$    but WITHOUT ANY WARRANTY; without even the implied warranty of
!!$    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!$    GNU General Public License for more details.
!!$
!!$    You should have received a copy of the GNU General Public License
!!$    along with MIM; if not, write to the Free Software
!!$    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
!!$    MA  02111-1307  USA

  !  User-defined parameters
  real, parameter :: PLEV=50.

  !  Declaration of variables - MODIFY to declare additional variables
  integer, dimension(:,:), pointer :: vtimer=>null()
  real, dimension(:,:), pointer :: uu=>null(), vv=>null(), uua=>null(), vva=>null()

  !  Get user-defined parameters                                        (use IO_Params)
  call getParams(input,output,gridFile,fileType)

  !  Initialization and opening                                         (use TopLevel)
  allocate(inOpen(size(input)))
  allocate(outOpen(size(output)))
  inOpen = .true. ; outOpen = .true.
  call openGridFiles(inOpen,outOpen,fileType)

  !  Set timer - MODIFY to fit time requirements                        (use TopLevel)
  timer => timeLoop('file',fileType,input(1))
  call validTimes(timer, fileType, vtimer)

  !  Get grid dimensions from primary input file                        (use TopLevel)
  call getGridSize(input(1),fileType,ni,nj,nk,ntimes,                   &
       mapFile=gridFile)

  !  Get file information (take lat/long from grid file)                (use TopLevel)
  call getGridInfo(input(1),fileType,ni,nj,nk,ntimes,lev,               &
       vCoord,fileTimer,labl,progType,dx,dy,mapProj,dgrw,infoFld, &
       gotLat,lat,long,gridFile,projParam)

  !  Construct dim                                                      (use Dimensions_Constructor)
  if (gotLat) then
     call makeDim(ni,nj,nk,dx,dy,lat,long,dxs,dys,lev,vCoord,infoFld,   &
	  fileType,progType,dim,mapProj=mapProj,projParam=projParam)
  else
     call makeDim(ni,nj,nk,dx,dy,lev,vCoord,infoFld,fileType,progType,	&
	  dim)
  endif

  !  Loop over time for read/write - MODIFY reads/calculations/writes
  do t=1,size(timer,dim=1)

     !  Read in input variable                                             (use FileIO)
     call readGrid(input(1),'UU',PLEV,timer(t,:),dim,uu)
     call readGrid(input(1),'VV',PLEV,timer(t,:),dim,vv)
     call readGrid(input(2),'UU',PLEV,vtimer(t,:),dim,uua)
     call readGrid(input(2),'VV',PLEV,vtimer(t,:),dim,vva)

     !  Write out output variable                                          (use FileIO)
     call writeGrid(output(1),uu-uua,'UU',PLEV,timer(t,:),dim)
     call writeGrid(output(1),vv-vva,'VV',PLEV,timer(t,:),dim)

     !  End of time loop
  enddo

  !  Close input/output files                                           (use TopLevel)
  call closeGridFiles(inOpen,outOpen,fileType)

  !  End of main program
end program process
