fld.in <- 'KE_50_mb'
fcst <- '072'
ctl.name <- 'e19y15v52a17'
exp.name <- 'jing077ete'
ext <- '_errors'
spec.path <- '../spec/'
ctl.name.spec <- paste0(spec.path, ctl.name)
exp.name.spec <- paste0(spec.path, exp.name)

ctl <- c()
exp <- c()
ctl.spec <- c()
exp.spec <- c()
for (fn in list.files(paste0(ctl.name, ext, '/', fld.in), pattern=paste0(".*_ke_.*_", fcst, ".harmon"))){
    ctl <- rbind(ctl, scan(paste0(ctl.name, ext, '/', fld.in, '/', fn), quiet=TRUE)[-1])
    exp <- rbind(exp, scan(paste0(exp.name, ext, '/', fld.in, '/', fn), quiet=TRUE)[-1])
    ctl.spec <- rbind(ctl.spec, scan(paste(ctl.name.spec, fld.in, fn, sep='/'), quiet=TRUE)[-1])
    exp.spec <- rbind(exp.spec, scan(paste(exp.name.spec, fld.in, fn, sep='/'), quiet=TRUE)[-1])
}

ctl.mean <- apply(ctl, 2, mean)
exp.mean <- apply(exp, 2, mean)
ctl.spec.mean <- apply(ctl.spec, 2, mean)
exp.spec.mean <- apply(exp.spec, 2, mean)
wn <- seq(1, ncol(ctl))

library(boot)
fcstmean <- function(x, v){
    return(mean(x[v]))
}
compute.ci <- function(x){
    ci.out <- c()
    for (j in 1:ncol(x)){
        ci.out <- rbind(ci.out, boot.ci(boot(x[,j], fcstmean, R=1000), type='perc')$percent[4:5])
    }
    return(ci.out)
}
ctl.ci <- compute.ci(ctl)
exp.ci <- compute.ci(exp)
ctl.spec.ci <- compute.ci(ctl.spec)
exp.spec.ci <- compute.ci(exp.spec)

p.width <- 10
p.height <- p.width/1.618
pdf(paste0('gdps_errspec',fld.in,'_',fcst,'.pdf'), paper='special', width=p.width,
    height=p.height, bg='white')
default <- par(mar=c(5,7,4,2))
plot(1, 1, xlim=c(1, 1024), ylim=c(1e-8, 1e2), type='n', xaxt='n', log='xy', yaxt='n',
     main='Energy Spectrum at 50 hPa (72h)', 
     xlab='Spherical Wavenumber', ylab='',
     cex.axis=1.5, cex.lab=1.5, cex.main=1.5)
axis(1, at=2^seq(0, 10), cex.axis=1.5)
axis(2, at=10^seq(-8, 2), cex.axis=1.5, las=1)
mtext(expression('Spectral Energy Density (m'^{2}*' s'^{-2}*')'), 2, line=4.8, cex=1.5)
polygon(c(wn, rev(wn)), c(ctl.ci[,1], rev(ctl.ci[,2])),
        col=adjustcolor('blue', alpha.f=0.4), border=NA)
polygon(c(wn, rev(wn)), c(exp.ci[,1], rev(exp.ci[,2])),
        col=adjustcolor('red', alpha.f=0.4), border=NA)
polygon(c(wn, rev(wn)), c(ctl.spec.ci[,1], rev(ctl.spec.ci[,2])),
        col=adjustcolor('blue', alpha.f=0.4), border=NA)
polygon(c(wn, rev(wn)), c(exp.spec.ci[,1], rev(exp.spec.ci[,2])),
        col=adjustcolor('red', alpha.f=0.4), border=NA)
lines(wn, ctl.spec.mean, type='l', col='blue', lwd=3, lty=1)
lines(wn, exp.spec.mean, type='l', col='red', lwd=3, lty=1)
lines(wn, ctl.mean, type='l', col='blue', lwd=3, lty=2)
lines(wn, exp.mean, type='l', col='red', lwd=3, lty=2)
legend('bottomleft', inset=0.02, legend=c('GEM Control (Energy)', 'OFFB51 (Energy)',
                                          'GEM Control (Error)', 'OFFB51 (Error)'),
       col=c('blue', 'red', 'blue', 'red'), lwd=3, lty=c(1,1,2,2), cex=1.4)
xo <- 50
xf <- 400
yo <- 0.07
yf <- exp( log(yo) + (-5/3) * (log(xf) - log(xo)) )
lines(c(xo,xf), c(yo,yf), lwd=2)
text(155, 0.03, labels='-5/3', cex=1.3)
xo <- 12
xf <- 45
yo <- 5
yf <- exp( log(yo) + (-3) * (log(xf) - log(xo)) )
lines(c(xo,xf), c(yo,yf), lwd=2)
text(25, 1.5, labels='-3', cex=1.3)

par(default)
dev.off()
