fld.in <- 'KE_50_mb'
fcst <- '072'
ctl.name <- 'e19y15v52a17'
exp.name <- 'jing077ete'
ext <- '_errors'
spec.path <- '../spec/'
ctl.name.spec <- paste0(spec.path, ctl.name)
exp.name.spec <- paste0(spec.path, exp.name)

ctl <- c()
exp <- c()
ctl.spec <- c()
exp.spec <- c()
for (fn in list.files(paste0(ctl.name, ext, '/', fld.in), pattern=paste0(".*_ke_.*_", fcst, ".harmon"))){
    ctl <- rbind(ctl, scan(paste0(ctl.name, ext, '/', fld.in, '/', fn), quiet=TRUE)[-1])
    exp <- rbind(exp, scan(paste0(exp.name, ext, '/', fld.in, '/', fn), quiet=TRUE)[-1])
    ctl.spec <- rbind(ctl.spec, scan(paste(ctl.name.spec, fld.in, fn, sep='/'), quiet=TRUE)[-1])
    exp.spec <- rbind(exp.spec, scan(paste(exp.name.spec, fld.in, fn, sep='/'), quiet=TRUE)[-1])
}

ctl.ratio <- ctl / ctl.spec
exp.ratio <- exp / exp.spec
ctl.mean <- apply(ctl.ratio, 2, mean)
exp.mean <- apply(exp.ratio, 2, mean)
wn <- seq(1, ncol(ctl))

library(boot)
fcstmean <- function(x, v){
    return(mean(x[v]))
}
compute.ci <- function(x){
    ci.out <- c()
    for (j in 1:ncol(x)){
        ci.out <- rbind(ci.out, boot.ci(boot(x[,j], fcstmean, R=1000), type='perc')$percent[4:5])
    }
    return(ci.out)
}
ctl.ci <- compute.ci(ctl.ratio)
exp.ci <- compute.ci(exp.ratio)

p.width <- 10
p.height <- p.width/1.618
pdf(paste0('gdps_errratio',fld.in,'_',fcst,'.pdf'), paper='special', width=p.width,
    height=p.height, bg='white')
default <- par(mar=c(5,7,4,2))
plot(1, 1, xlim=c(1, 1024), ylim=c(0, 3), type='n', xaxt='n', log='x', las=1,
     main='Relative Error Spectrum at 50 hPa (72h)', 
     xlab='Spherical Wavenumber', ylab='',
     cex.axis=1.5, cex.lab=1.5, cex.main=1.5,
     panel.first={
         abline(h=1, col='grey', lwd=2, lty=2);
     })
axis(1, at=2^seq(0, 10), cex.axis=1.5)
mtext('Relative Error', 2, line=4.8, cex=1.5)
polygon(c(wn, rev(wn)), c(ctl.ci[,1], rev(ctl.ci[,2])),
        col=adjustcolor('blue', alpha.f=0.4), border=NA)
polygon(c(wn, rev(wn)), c(exp.ci[,1], rev(exp.ci[,2])),
        col=adjustcolor('red', alpha.f=0.4), border=NA)
lines(wn, ctl.mean, type='l', col='blue', lwd=3, lty=1)
lines(wn, exp.mean, type='l', col='red', lwd=3, lty=1)
legend('topleft', inset=0.02, legend=c('GEM Control', 'OFFB51'),
       col=c('blue', 'red'), lwd=3, cex=1.4)
par(default)
dev.off()
