range.max <- 240

ctl <- read.table('e19y15v52a17_zonwind.dat')
offb51 <- read.table('jing077ete_zonwind.dat')
obs <- read.table('analysis_zonwind.dat')

dep <- function(x){
    dep.out <- c()
    for (i in 1:nrow(x)){
        dep.out <- rbind(dep.out, x[i,]-x[i,1])
    }
    return(as.matrix(dep.out))
}

# Compute derived quantities to plot
ctl.dep <- dep(ctl)
offb51.dep <- dep(offb51)
obs.dep <- dep(obs)
model.time <- seq(0, range.max, length.out=ncol(ctl))
obs.time <- seq(0, range.max, length.out=ncol(obs))
mask <- obs.time %in% model.time

# Bootstrap for confidence intervals
library(boot)
fcstmean <- function(x, v){
    return(mean(x[v]))
}
compute.ci <- function(x){
    ci.out <- c(0, 0)
    for (j in 2:ncol(x)){
        ci.out <- rbind(ci.out, boot.ci(boot(x[,j], fcstmean, R=1000), type='perc')$percent[4:5])
    }
    return(ci.out)
}
ctl.ci <- compute.ci(ctl.dep)
offb51.ci <- compute.ci(offb51.dep)


p.width <- 9
p.height <- p.width/1.618
pdf('gdps_polarvort.pdf', paper='special', width=p.width, height=p.height, bg='white')
default <- par(mar=c(5,5,4,2))
plot(1, 1, xlim=c(0, 240), ylim=c(-15, 10), type='n', xaxt='n', xaxs='i',
     main=expression('Forecast Change in 1 hPa Zonal-Mean Zonal Wind at 60'^o*'S'), 
     las=1, xlab='Forecast (h)', ylab=expression('Zonal Wind Speed Change (m s'^{-1}*')'),
     cex.axis=1.6, cex.lab=1.6, cex.main=1.6, 
     panel.first={
         abline(h=0, col='grey', lwd=4, lty=2);
     })
axis(1, at=model.time, cex.axis=1.6)
polygon(c(model.time, rev(model.time)), c(ctl.ci[,1], rev(ctl.ci[,2])),
        col=adjustcolor('blue', alpha.f=0.4), border=NA)
polygon(c(model.time, rev(model.time)), c(offb51.ci[,1], rev(offb51.ci[,2])),
        col=adjustcolor('red', alpha.f=0.4), border=NA)
lines(model.time, apply(ctl.dep, 2, mean), type='l', col='blue', lwd=3)
lines(model.time, apply(offb51.dep, 2, mean), type='l', col='red', lwd=3)
lines(obs.time[mask], apply(obs.dep[,mask], 2, mean), type='l', col='black', lwd=3)
legend('bottomleft', inset=0.02, legend=c('ERA5', 'GEM Control', 'OFFB51'),
       col=c('black', 'blue', 'red'), lwd=3, cex=1.4)
par(default)
dev.off()
