fld.names <- c('pres', 'wind', 'windm')
ctl.name <- 'e19y15v52a17'
exp.name <- 'jing077ete'
plot.range <- 240

tcmean <- function(x, v){
    return(mean(x[v]))
}

library(boot)
for (fld.name in fld.names){
       
    if (fld.name == 'pres'){
        mult <- 1
        name.long <- 'Pressures'
        name.short <- fld.name
        units.y <- 'Central Pressure Bias (hPa)'
        yrange <- c(-15, 15)
        legend.pos <- 'bottomleft'
        ctl.n.pos <- max(yrange)
        exp.n.pos <- ctl.n.pos - 1.6
    } else if (fld.name == 'wind'){
        mult <- 0.5144
        name.long <- 'Winds'
        name.short <- fld.name
        units.y <- expression('Wind Speed Bias (m s'^{-1}*')')
        yrange <- c(-15, 5)
        legend.pos <- 'topleft'
        ctl.n.pos <- min(yrange)
        exp.n.pos <- ctl.n.pos + 1.6 * (2/3)
    } else if (fld.name == 'windm'){
        mult <- 0.5144
        name.long <- 'Winds'
        name.short <- 'wind'
        units.y <- expression('Wind Speed Bias (m s'^{-1}*')')
        yrange <- c(-40, -5)
        legend.pos <- 'topleft'
        ctl.n.pos <- min(yrange)
        exp.n.pos <- ctl.n.pos + 1.6 * (7/6)
    }
  
    ctl.bias <- c()
    exp.bias <- c()
    ctl.ci <- c()
    exp.ci <- c()
    ctl.n <- c()
    exp.n <- c()
    fcsts <- seq(0, plot.range, by=6)
    for (fcst in fcsts){
        ctl.d <- mult * scan(paste0('data/',ctl.name,'_scatter_',name.short,'_',name.short,'_',fcst,'h.txt'), quiet=TRUE)
        exp.d <- mult * scan(paste0('data/',exp.name,'_scatter_',name.short,'_',name.short,'_',fcst,'h.txt'), quiet=TRUE)
        ctl.err <- ctl.d[seq(1, length(ctl.d), by=2)] - ctl.d[seq(2, length(ctl.d), by=2)]
        exp.err <- exp.d[seq(1, length(exp.d), by=2)] - exp.d[seq(2, length(exp.d), by=2)]
        if (fld.name == 'windm'){
            ctl.err <- ctl.err[ctl.d[seq(2, length(ctl.d), by=2)] >= 33.5]
            exp.err <- exp.err[exp.d[seq(2, length(exp.d), by=2)] >= 33.5]
        }
        ctl.bias <- c(ctl.bias, mean(ctl.err))
        exp.bias <- c(exp.bias, mean(exp.err))
        ctl.ci <- rbind(ctl.ci, boot.ci(boot(ctl.err, tcmean, R=1000), type='perc')$percent[4:5])
        exp.ci <- rbind(exp.ci, boot.ci(boot(exp.err, tcmean, R=1000), type='perc')$percent[4:5])
        ctl.n <- c(ctl.n, length(ctl.err))
        exp.n <- c(exp.n, length(exp.err))
    }
    
    pdf(paste0('gdps_series_',fld.name,'.pdf'),
        paper='special', height=7, width=9, bg='white')
    default <- par(mar=c(5,5,4,2))
    plot(1,1, type='n', xlim=c(0, plot.range), ylim=yrange, xaxt='n',
         cex.main=1.6, cex.lab=1.6, cex.axis=1.6,
         main=paste0('Global Tropical Cyclone ',name.long),
         xlab="Forecast (h)", ylab=units.y,
         panel.first={
             abline(h=0, col='grey', lwd=2, lty=2);
         })
    fcst.main <- seq(0, plot.range, by=24)
    axis(1, at=fcst.main, cex.axis=1.6)
    polygon(c(fcsts, rev(fcsts)), c(ctl.ci[,1], rev(ctl.ci[,2])),
            col=adjustcolor('blue', alpha.f=0.4), border=NA)
    polygon(c(fcsts, rev(fcsts)), c(exp.ci[,1], rev(exp.ci[,2])),
            col=adjustcolor('red', alpha.f=0.4), border=NA)
    lines(fcsts, ctl.bias, col='blue', lwd=3)
    lines(fcsts, exp.bias, col='red', lwd=3)
    text(fcst.main, ctl.n.pos, labels=ctl.n[fcsts %in% fcst.main], cex=1.2, col='blue')
    text(fcst.main, exp.n.pos, labels=exp.n[fcsts %in% fcst.main], cex=1.2, col='red')
    legend(legend.pos, inset=0.02, legend=c('GEM Control', 'OFFB51'),
       col=c('blue', 'red'), lwd=3, cex=1.4)
    par(default)
    dev.off()
}
