flds <- c('TT', 'GZ', 'UU', 'HU')
sig.threshold <- 95
y.log <- FALSE

for (fld in flds){
    fld.mult <- 1.
    fld.lwd <- 3
    if (fld == 'TT'){
        fld.name <- 'Temperature'
        fld.unit <- paste(fld.name, '(K)')
    } else if (fld == 'UU'){
        fld.name <- 'Zonal Wind'
        fld.unit <- expression('Zonal Wind (m s'^{-1}*')')
    } else if (fld == 'HU'){
        fld.name <- 'Specific Humidity'
        fld.unit <- expression('Specific Humidity (g kg'^{-1}*')')
    } else if (fld == 'GZ'){
        fld.name <- 'Geopotential Height'
        fld.unit <- paste(fld.name, '(m)')
        fld.mult <- 10.
    }

    d <- read.table(paste0('PRO_ALL_120_ALL_ALL_monde_e19y15v52a17-jing077ete_', tolower(fld), '.txt'), header=TRUE)
    fld.range <- c(min(d$biasexp2), max(d$stdexp2))
    if (y.log){
        ext <- ''
    } else {
        ext <- '_lin'
    }
    postscript(paste0('prof_120h_world-', fld, ext, '.ps'), bg='white',
               paper='special', width=7, height=9, horiz=FALSE)
    default <- par(mar=c(5,5,4,2))
    plot(1, 1, type='n', xlim=fld.range, ylim=rev(range(d$Niveau)),
         xlab=fld.unit, ylab=NA, yaxt='n', ylog=y.log, yaxs='i',
         cex.lab=1.6, cex.axis=1.6,
         panel.first={
             abline(v=0, col='grey', lty=2, lwd=2);
         })
    if (y.log){
        axis(2, at=c(1000, 700, 500, 250, 100, 50, 30, 20, 10, 7, 5, 3, 2, 1), cex.axis=1.6, las=1)
    } else {
        axis(2, at=c(1000, 925, 850, 700, 600, 500, 400, 300, 250, 150, 100, 50, 1), cex.axis=1.6, las=1)
    }
    mtext('Pressure (hPa)', 2, line=3.5, cex=1.6)
    lines(d$biasexp1, d$Niveau, lty=2, col='blue', lwd=fld.lwd)
    lines(d$biasexp2, d$Niveau, lty=2, col='red', lwd=fld.lwd)
    lines(d$stdexp1, d$Niveau, lty=1, col='blue', lwd=fld.lwd)
    lines(d$stdexp2, d$Niveau, lty=1, col='red', lwd=fld.lwd)
    for (i in 1:nrow(d)){
        if (d$biasconf[i] >= sig.threshold){
            if (abs(d$biasexp2[i]) < abs(d$biasexp1[i])){
                pt.x <- d$biasexp2[i]
                pt.col <- 'red'
            } else {
                pt.x <- d$biasexp1[i]
                pt.col <- 'blue'
            }
            points(pt.x, d$Niveau[i], pch=19, col=pt.col, cex=1.8)
        }
        if (d$stdconf[i] >= sig.threshold){
            if (d$stdexp2[i] < d$stdexp1[i]){
                pt.x <- d$stdexp2[i]
                pt.col <- 'red'
            } else {
                pt.x <- d$stdexp1[i]
                pt.col <- 'blue'
            }
            points(pt.x, d$Niveau[i], pch=19, col=pt.col, cex=1.8)
        }
    }
    if (fld == 'HU'){
        legend('topright', inset=0.05, legend=c('GEM Control', 'OFFB51'),
               col=c('blue', 'red'), lwd=3, cex=1.4)
    }
    par(default)
}
