karman <- 0.4
height <- 10

zo <- function(type, ustar=NA){
    am <- 0.11
    ach <- 0.018
    g <- 9.81
    visc <- 1.5e-5
    if (is.na(ustar[1])) {ustar <- get.ust(type)}
    if (type == 'gem'){
        zo.min <- 1.5e-5
        zo.max <- 5e-3
        zo.c <- am * visc / pmax(ustar, 0.01) + ach * ustar^2 / g
        return(pmin(pmax(zo.c, zo.min), zo.max))
    } else if (type == 'ecmwf.ayrton'){
        zo.min.ecmwf <- 1e-5
        zo.max.ecmwf <- 5e-2
        beta1 = 0.018
        acr = 0.64
        cl = 10.
        uec = acr/karman
        due = (karman)^2*uec/(log(cl/beta1))
        aux1 = beta1 + (ustar - 0.4)*(0.06 - beta1)/(1.4 - 0.4)
        aux1 = pmin(1.,pmax(0.01,aux1))
        aux2 = 0.5*(1. - tanh(0.5*karman*(ustar - uec)/due))
        ach.ecmwf = aux1 * aux2
        zo.c <- am * visc / pmax(ustar, 0.01) + ach.ecmwf * ustar^2 / g
        return(pmin(pmax(zo.c, zo.min.ecmwf), zo.max.ecmwf))
    } else if (type == 'ecmwf.ron'){
        ustar.c = 0.64 / karman
        d.ustar = karman^2 * ustar.c / 15
        zo.min.ecmwf <- 9e-5
        zo.max.ecmwf <- 7e-3
        aux1 = ach + (ustar - 0.5) * (0.06 - ach) / (1.4 - 0.4)
        aux1 = pmin(1., pmax(0.01, aux1))
        aux2 = 0.5 * (1. - tanh(0.5*karman * (ustar - 1.1*ustar.c) / d.ustar))
        ach.ecmwf = aux1 * aux2
        zo.c <- am * visc / pmax(ustar, 0.01) + ach.ecmwf * ustar^2 / g
        return(pmin(pmax(zo.c, zo.min.ecmwf), zo.max.ecmwf))
    } else if (type == 'ecmwf.rpnphy'){
        z0_min <- 9e-5
        z0_max <- 7e-3
        ustar_c = 0.64 / karman
        d_ustar = karman^2. * ustar_c / 15.
        c1 = 0.018 + 0.042 * (ustar - 0.5)
        c1 = pmin(1., pmax(0.01, c1))
        c2 = 0.5 * (1. - tanh(0.5*karman * (ustar - 1.1 * ustar_c) / d_ustar))
        beta_ch = c1 * c2
        zo.c <- am * visc / pmax(ustar, 0.01) + beta_ch * ustar^2 / g
        return(pmin(pmax(zo.c, z0_min), z0_max))
    } else if (type == 'wrf1'){
        z1 <- 0.011 * ustar^2 / g + 1.59e-5
        z2 <- 10 / exp(9.5*ustar^{-1/3}) + 1.65e-6 / (pmax(ustar, 0.01))
        zw <- pmin(1, (ustar / 1.06)^0.3)
        zo.c <- pmax(1.27e-7, pmin(zw*z2 + (1-zw)*z1, 2.85e-3))
        return(zo.c)
    }
}

zot <- function(type, ustar=NA){
    if (is.na(ustar[1])) {ustar <- get.ust(type)}
    if (type == 'gem'){
        zo.c <- zo(type, ustar)
        return(pmin(2.e-5 / pmax(ustar, 0.01), 1.e-4))
    } else if (type == 'wrf1'){
        return(rep(1e-4, times=length(ustar)))
    }
}

un <- function(type, ustar=NA){
    if (is.na(ustar[1])){ustar <- get.ust(type)}
    return(ustar / karman * log(1 + height / zo(type, ustar=ustar)))
}

cd <- function(type, ustar=NA){
    if (is.na(ustar[1])){ustar <- get.ust(type)}
    return(karman^2 / log(height/zo(type, ustar=ustar))^2)
}

ct <- function(type, ustar=NA){
    if (is.na(ustar[1])){ustar <- get.ust(type)}
    return(sqrt(cd(type, ustar)) * karman / log(height/zot(type, ustar=ustar)))
}

get.ust <- function(type, v10.lim=80, inc=0.01){
    ust <- c(inc)
    v10 <- 0
    while (v10 < v10.lim){
        ust.new <- ust[length(ust)] + inc
        ust <- c(ust, ust.new)
        v10 <- un(type, ustar=ust.new)
    }
    return(ust)
}

paper.width <- 9
paper.height <- 7
plot.margins <- c(5,7,4,2)
postscript('rough_zo.ps', horiz=FALSE, bg='white',
           paper='special', width=paper.width, height=paper.height)
default <- par(mar=plot.margins)
plot(1,1, type='n', log='y', xlim=c(0, 80), ylim=c(1e-8, 1e-1),
     cex.axis=1.6, cex.lab=1.6, cex.main=1.6, las=1, xaxs='i',
     main="Roughness Length Parameterizations", ylab='',
     xlab=expression("10-m Wind Speed (m s"^{-1}*")"))
mtext("Roughness Length (m)", 2, cex=1.6, line=5)
lines(un('wrf1'), zo('wrf1'), lwd=2)
lines(un('wrf1'), zot('wrf1'), lwd=2, lty=2)
lines(un('gem'), zo('gem'), lwd=5, col='blue')
lines(un('gem'), zot('gem'), lwd=5, col='blue', lty=2)
legend('bottomleft', inset=0.02, cex=1.4, bg='white',
       legend=c('WRF momentum roughness', 'WRF scalar roughness',
                'GEM momentum roughness', 'GEM scalar roughness'),
       lty=c(1, 2, 1, 2), col=c('black', 'black', 'blue', 'blue'), lwd=c(2, 2, 5, 5))
par(default)
dev.off()

postscript('rough_cd.ps', horiz=FALSE, bg='white',
           paper='special', width=paper.width, height=paper.height)
mult <- 1000
default <- par(mar=plot.margins)
plot(1,1, type='n', xlim=c(0, 80), ylim=c(0, 3),
     cex.axis=1.6, cex.lab=1.6, cex.main=1.6, las=1, xaxs='i',
     main="Exchange Coefficient Parameterizations", ylab='',
     xlab=expression("10-m Wind Speed (m s"^{-1}*")"))
mtext(expression("Exchange Coefficient (x10"^{-3}*")"), 2, cex=1.6, line=4.8)
lines(un('wrf1'), mult*cd('wrf1'), lwd=2)
lines(un('wrf1'), mult*ct('wrf1'), lwd=2, lty=2)
lines(un('gem'), mult*cd('gem'), lwd=5, col='blue')
lines(un('gem'), mult*ct('gem'), lwd=5, col='blue', lty=2)
legend('bottomleft', inset=0.02, cex=1.4, bg='white',
       legend=c('WRF drag coefficient', 'WRF scalar coefficient',
                'GEM drag coefficient', 'GEM scalar coefficient'),
       lty=c(1, 2, 1, 2), col=c('black', 'black', 'blue', 'blue'), lwd=c(2, 2, 5, 5))
par(default)
dev.off()
