karman <- 0.4

zo <- function(ustar, type='gem'){
    am <- 0.11
    ach <- 0.018
    g <- 9.81
    visc <- 1.5e-5
    if (type == 'gem'){
        zo.min <- 1.5e-5
        zo.max <- 5e-3
        zo.c <- am * visc / pmax(ustar, 0.01) + ach * ustar^2 / g
        return(pmin(pmax(zo.c, zo.min), zo.max))
    } else if (type == 'ecmwf.ayrton'){
        zo.min.ecmwf <- 1e-5
        zo.max.ecmwf <- 5e-2
        beta1 = 0.018
        acr = 0.64
        cl = 10.
        uec = acr/karman
        due = (karman)^2*uec/(log(cl/beta1))
        aux1 = beta1 + (ustar - 0.4)*(0.06 - beta1)/(1.4 - 0.4)
        aux1 = pmin(1.,pmax(0.01,aux1))
        aux2 = 0.5*(1. - tanh(0.5*karman*(ustar - uec)/due))
        ach.ecmwf = aux1 * aux2
        zo.c <- am * visc / pmax(ustar, 0.01) + ach.ecmwf * ustar^2 / g
        return(pmin(pmax(zo.c, zo.min.ecmwf), zo.max.ecmwf))
    } else if (type == 'ecmwf.ron'){
        ustar.c = 0.64 / karman
        d.ustar = karman^2 * ustar.c / 15
        zo.min.ecmwf <- 9e-5
        zo.max.ecmwf <- 7e-3
        aux1 = ach + (ustar - 0.5) * (0.06 - ach) / (1.4 - 0.4)
        aux1 = pmin(1., pmax(0.01, aux1))
        aux2 = 0.5 * (1. - tanh(0.5*karman * (ustar - 1.1*ustar.c) / d.ustar))
        ach.ecmwf = aux1 * aux2
        zo.c <- am * visc / pmax(ustar, 0.01) + ach.ecmwf * ustar^2 / g
        return(pmin(pmax(zo.c, zo.min.ecmwf), zo.max.ecmwf))
    } else if (type == 'ecmwf.rpnphy'){
        z0_min <- 9e-5
        z0_max <- 7e-3
        ustar_c = 0.64 / karman
        d_ustar = karman^2. * ustar_c / 15.
        c1 = 0.018 + 0.042 * (ustar - 0.5)
        c1 = pmin(1., pmax(0.01, c1))
        c2 = 0.5 * (1. - tanh(0.5*karman * (ustar - 1.1 * ustar_c) / d_ustar))
        beta_ch = c1 * c2
        zo.c <- am * visc / pmax(ustar, 0.01) + beta_ch * ustar^2 / g
        return(pmin(pmax(zo.c, z0_min), z0_max))
    }
        
}

un <- function(ustar, type){
    return(ustar / karman * log(1 + height / zo(ustar, type)))
}

cd <- function(ustar, type){
    return((ustar / un(ustar, type))^2)
}

ust <- seq(0, 2.5, by=0.01)
type <- 'ecmwf.ayrton'

plot(un(ust, type), zo(ust, type), type='l')
plot(un(ust, type), cd(ust, type), type='l', ylim=c(0, 0.004))
ec.u10 <- seq(0, 80, by=5)
ec.cd <- c(3, 1.2, 1.5, 1.8, 2.4, 3, 3, 2.8, 2, 1.75, 1.5, 1.3, 1.25, 1.2, 1.2, 1.2, 1.2)
points(ec.u10, 0.001*ec.cd, type='p')

plot(ust, cd(ust, type), type='l',ylim=c(0,0.01))
