program process

  !  Associate required modules - MODIFY to include additional modules
  use Std_Variables, only: &
       ni,nj,nk,ntimes,t,progType,fileType,labl,dx,dy,dgrw,dim                  !variables
  use IO_Params, only: &
       input,output,gridFile,infoFld, &                                         !variables
       getParams                                                                !subprograms
  use Dimensions_Constructor, only: &
       dxs,dys, &                                                               !variables
       makeDim                                                                  !subprograms
  use TopLevel, only: &
       inOpen,outOpen,timer,fileTimer,projParam,lat,long,lev,vCoord,mapProj,gotLat,&!variables
       openGridFiles,closeGridFiles,getGridSize,getGridInfo,timeLoop            !subprograms
  use FileIO, only: &
       readGrid,writeGrid                                                       !subprograms
  use FileInfo_Type
  use Dimensions_Type
  use FST_File_Utilities
  use String
  use Math
! use Interface_MRG

! load FST       <- force comp90.d loading of ARMNLIB (FST library)
! load GEMPAK    <- force comp90.d loading of GEMLIB (GEMPAK library)
! load CDF       <- force comp90.d loading of NETCDF (netCDF library)

  !  This program template serves as the basis for the developer's
  !  interface for the MRG Interactive MetCal program and library.  
  !  Simple setup and IO operations are performed here; the developer
  !  is expected to extend upon this code as needed during further
  !  subprogram development for the library.

  implicit none

  !  Program Name: 
  !  Version: 1.0
  !  Programmer: 
  !  Date Modified:
  !  Modified by:

!!$    MRG_Interactive MetCal metorological calculator - developer's template
!!$    Copyright 2001 (C) Ron McTaggart-Cowan
!!$
!!$    This file is part of MRG Interactive MetCal (MIM).
!!$
!!$    MIM is free software; you can redistribute it and/or modify
!!$    it under the terms of the GNU General Public License as published by
!!$    the Free Software Foundation; either version 2 of the License or
!!$    any later version.
!!$
!!$    MIM is distributed in the hope that it will be useful,
!!$    but WITHOUT ANY WARRANTY; without even the implied warranty of
!!$    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!$    GNU General Public License for more details.
!!$
!!$    You should have received a copy of the GNU General Public License
!!$    along with MIM; if not, write to the Free Software
!!$    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
!!$    MA  02111-1307  USA

  !  User-defined parameters
  real, parameter :: MAXRAD=500000.
  
  !  Declaration of variables - MODIFY to declare additional variables
  integer :: i,j,fcst,tci,tcj,imin,imax,jmin,jmax
  integer, dimension(2) :: pos
  real :: tclat,tclon,minslp
  real, dimension(:,:), allocatable :: p0f
  real, dimension(:,:), pointer :: p0=>null()
  character(len=3) :: cfcst
  character(len=1024) :: fbase,fn
  logical :: file_exists

  !  Get user-defined parameters                                        (use IO_Params)
  call getParams(input,output,gridFile,fileType)
  fbase = input(1)%fileName
  input(1)%fileName = trim(fbase)//'000'

  !  Initialization and opening                                         (use TopLevel)
  allocate(inOpen(size(input)))
  allocate(outOpen(size(output)))
  inOpen = .true. ; outOpen = .false.
  call openGridFiles(inOpen,outOpen,fileType)
  open(output(1)%unit, file=output(1)%fileName)

  !  Get grid dimensions from primary input file                        (use TopLevel)
  call getGridSize(input(1),fileType,ni,nj,nk,ntimes,                   &
    mapFile=gridFile)

  !  Set space
  allocate(p0f(ni,nj))
  
  !  Get file information (take lat/long from grid file)                (use TopLevel)
  call getGridInfo(input(1),fileType,ni,nj,nk,ntimes,lev,               &
             vCoord,fileTimer,labl,progType,dx,dy,mapProj,dgrw,infoFld, &
             gotLat,lat,long,gridFile,projParam)

  !  Construct dim                                                      (use Dimensions_Constructor)
  if (gotLat) then
     call makeDim(ni,nj,nk,dx,dy,lat,long,dxs,dys,lev,vCoord,infoFld,   &
	  fileType,progType,dim,mapProj=mapProj,projParam=projParam)
  else
     call makeDim(ni,nj,nk,dx,dy,lev,vCoord,infoFld,fileType,progType,	&
	  dim)
  endif

  !  Loop over inputs
  fcst = 0
  write(cfcst, '(i3)') fcst
  cfcst = lpad(cfcst, '0')
  fn = trim(fbase)//cfcst
  inquire(file=fn, exist=file_exists)
  do while (file_exists)

     !  Open input file
     call closeFST(input(1))
     input(1)%fileName = fn
     call openFST(input(1), type='input')
     
     !  Read in input variable                                             (use FileIO)
     call readGrid(input(1),'P0',0.,(/-1,-1,-1/),dim,p0)
     p0f(:,:) = p0(:,:)
     do i=1,5
        call filter(p0f, dim)
     enddo
     if (fcst == 0) then
        pos = minloc(p0f)
        tclat = dim%lat(pos(1),pos(2))
        tclon = dim%long(pos(1),pos(2))
     endif        
     
     !  Calculations here...
     pos = minloc(p0f)
     SLOW_SEARCH: if (pathLength(tclat, tclon, dim%lat(pos(1),pos(2)), dim%long(pos(1),pos(2))) > &
          MAXRAD) then
        minslp = maxval(p0f)
        call grptLocator(tclat, tclon, tci, tcj, dim)
        imin = max(1, tci - ceiling(1.1*MAXRAD/dim%dx))
        imax = min(dim%ni, tci + ceiling(1.1*MAXRAD/dim%dx))
        jmin = max(1, tcj - ceiling(1.1*MAXRAD/dim%dy))
        jmax = min(dim%nj, tcj + ceiling(1.1*MAXRAD/dim%dy))
        do j=jmin,jmax
           do i=imin,imax
              if (pathLength(tclat, tclon, dim%lat(i,j), dim%long(i,j)) < MAXRAD) then
                 if (p0f(i,j) < minslp) then
                    minslp = p0f(i,j)
                    pos(1) = i
                    pos(2) = j
                 endif
              endif
           enddo
        enddo
     endif SLOW_SEARCH
     tclat = dim%lat(pos(1),pos(2))
     tclon = dim%long(pos(1),pos(2))
     write(output(1)%unit, '(i,x,2(f,x),2(i,x),f)') fcst, tclat, tclon, pos(1), &
          pos(2), p0(pos(1),pos(2))

     !  Generate next file name
     file_exists = .false.
     do while (.not. file_exists .and. fcst < 240) 
        fcst = fcst+1
        write(cfcst, '(i3)') fcst
        cfcst = lpad(cfcst, '0')
        fn = trim(fbase)//cfcst
        inquire(file=fn, exist=file_exists)
     enddo
        
  !  End of time loop
  enddo

  !  Close input/output files                                           (use TopLevel)
  call closeGridFiles(inOpen,outOpen,fileType)
  close(output(1)%unit)

  !  End of main program
end program process
