# User configuration
run<-'i5ipbl'
range<-240
radius<-3
fld<-'RT'
results.dir<-'results'

# Field-specific configuration
fld.mult <- 1
if (fld == 'V10T'){
    fld.name <- '10-m Tangential Wind'
    fld.range <- c(0, 50)
    fld.palette.func <- 'sequential_hcl'
    fld.col <- 'BluYl'
    fld.col.inc <- 2
    fld.units <- expression(m~s^-1)
} else if (fld == 'V10R'){
    fld.name <- '10-m Radial Wind'
    fld.range <- c(-30, 30)
    fld.palette.func <- 'diverging_hcl'
    fld.col <- 'Blue-Red'
    fld.col.inc <- 2
    fld.units <- expression(m~s^-1)
} else if (fld == 'FC'){
    fld.name <- 'Sensible Heat Flux'
    fld.range <- c(-500,500)
    fld.palette.func <- 'diverging_hcl'
    fld.col <- 'Blue-Red'
    fld.col.inc <- 50
    fld.units <- expression(W~m^-2)
} else if (fld == 'FV'){
    fld.name <- 'Latent Heat Flux'
    fld.range <- c(-1000, 1000)
    fld.palette.func <- 'diverging_hcl'
    fld.col <- 'Blue-Red'
    fld.col.inc <- 100
    fld.units <- expression(W~m^-2)
} else if (fld == 'FQ'){
    fld.name <- 'Surface Momentum Flux'
    fld.range <- c(0, 10)
    fld.palette.func <- 'sequential_hcl'
    fld.col <- 'Viridis'
    fld.col.inc <- .5
    fld.units <- expression(kg~m^-1~s^-2)
} else if (fld == 'P0'){
    fld.name <- 'Surface Pressure'
    fld.range <- c(1012, 1000)
    fld.palette.func <- 'sequential_hcl'
    fld.col <- 'Viridis'
    fld.col.inc <- 0.5
    fld.units <- 'hPa'
} else if (fld == 'RT'){
    fld.name <- 'Rain Rate'
    fld.range <- c(0, 50)
    fld.palette.func <- 'sequential_hcl'
    fld.col <- 'Viridis'
    fld.col.inc <- 2.5
    fld.units <- expression(mm~h^-1)
    fld.mult <- 1000 * 3600
} else {
    stop(paste("Error: Unknown field fld =",fld))
}

# Acquire packages
source("/home/rmt001/R/my.filled.contour.R")
require(colorspace)

# Retrieve coordinate vectors
coord.rad<-scan(paste0(results.dir,'/',run,'/coord_rad.dat'))
coord.rad.r<-rev(coord.rad)

# Plot 2D fields
d.raw <- c()
coord.t <- c()
for (i in 0:range){
    fcst <- sprintf("%03d", i)
    fname <- paste0(results.dir,'/',run,'/',fld,'/azmean_',fcst,'.dat')
    if (file.exists(fname)){
        d.raw<-rbind(d.raw,read.table(fname))
        coord.t <- c(coord.t, i)
    }
}
d.r<-d.raw[, order(ncol(d.raw):1)] * fld.mult

levs <- seq(min(fld.range), max(fld.range), by=fld.col.inc)
nlevs <- length(levs)
postscript(paste(run,'_',fld,'_azhov.ps',sep=''), paper='special',
           width=7, height=7, bg='white', horiz=FALSE)
col.pal <- get(fld.palette.func)(nlevs, palette=fld.col)
if (fld.range[1] > fld.range[2]){col.pal <- rev(col.pal)}
my.filled.contour(coord.rad.r, coord.t, t(d.r), zlim=fld.range, levels=levs,
                  xlim=c(0, radius), ylab='Lead Time (h)', xlab='Radius (deg)',
                  yaxt='n', col=col.pal,
                  cex.axis=1.4, cex.lab=1.4, main=fld.name,
                  plot.axes={
                      axis(1, cex.axis=1.4);
                      axis(2, at=seq(min(coord.t), max(coord.t), by=24),
                           cex.axis=1.4, las=2)},
                  key.title={
                      mtext(fld.units, side=3, las=0, line=2.5, cex=1.2, adj=0)},
                  key.axes={axis(4, cex.axis=1.2)})
dev.off()
