# User configuration
run<-'dg25ctl'
interval<-1
fcst.in<-240
fcst<-sprintf("%03d", fcst.in)
radius.default<-3.6
height.max.default<-16
flds<-c('UU','VT','VR','VTD','VRD','VTP','VRP','VTBL','VRBL','KM','WW','HU','HR','TPHY','TI')
results.dir<-'../results'

# Run-specific configuratoin
run.std <- sub('TC_idealized_520a19_', '', run)
if (run.std == 'dg25ctl'){
    run.name <- 'GEM Control'
} else if (run.std == 'dg25offb51'){
    run.name <- 'OFFB51'
} else {
    run.name <- run.std
}

# Loop over all fields to generate plots
fld.mult <- 1
for (fld in flds){

    # Defaults
    radius <- radius.default
    height.max <- height.max.default
    
    # Field-specific configurations
    if (fld == 'UU'){
        fld.name <- 'Wind Speed'
        fld.range <- c(0, 65)
#        fld.range <- c(-10, 15)
        fld.palette.func <- 'rainbow'
        fld.col <- 'Dark'
        fld.col.inc <- 2
#        fld.col.inc <- 1
        fld.lev <- 'm'
    } else if (fld == 'VT'){
        fld.name <- 'Tangential Wind'
        fld.range <- c(-10, 50)
#        fld.range <- c(-10, 15)
        fld.palette.func <- 'sequential_hcl'
        fld.col <- 'Viridis'
        fld.col.inc <- 2
#        fld.col.inc <- 1
        fld.lev <- 'm'
    } else if (fld == 'VR'){
        fld.name <- 'Radial Wind'
        fld.range <- c(-20, 20)
#        fld.range <- c(-5, 5)
        fld.palette.func <- 'diverging_hcl'
        fld.col <- 'Blue-Red'
        fld.col.inc <- 2
#        fld.col.inc <- .5
        fld.lev <- 'm'
    } else if (fld == 'WW'){
        fld.name <- 'Omega Vertical Motion'
        fld.range <- c(-10,10)
#        fld.range <- c(-2,2)
        fld.palette.func <- 'diverging_hcl'
        fld.col <- 'Blue-Red'
        fld.col.inc <- 1
#        fld.col.inc <- 0.2
        fld.lev <- 't'
    } else if (fld == 'KM'){
        fld.name <- 'PBL Momentum Diffusion Coefficient'
        fld.range <- c(0,200)
        fld.palette.func <- 'sequential_hcl'
        fld.col <- 'Viridis'
        fld.col.inc <- 10
        height.max <- 3
        radius <- 1.5
        fld.lev <- 't'
    } else if (fld == 'HR'){
        fld.name <- 'Relative Humidity'
        fld.range <- c(0,1)
        fld.palette.func <- 'sequential_hcl'
        fld.col <- 'Viridis'
        fld.col.inc <- 0.05
        fld.lev <- 't'
    } else if (fld == 'HU'){
        fld.name <- 'Specific Humidity'
        fld.range <- c(0,0.02)
        fld.palette.func <- 'sequential_hcl'
        fld.col <- 'Viridis'
        fld.col.inc <- 0.001
        fld.lev <- 't'
    } else if (fld == 'TI'){
        fld.name <- 'Longwave Heating'
        fld.mult <- 3600*24
        fld.range <- c(-10, 10)
        fld.palette.func <- 'diverging_hcl'
        fld.col <- 'Blue-Red'
        fld.col.inc <- 1
        fld.lev <- 't'
    } else if (fld == 'TPHY'){
        fld.name <- 'Physics Heating'
        fld.mult <- 3600*24
        fld.range <- c(-20, 20)
        fld.palette.func <- 'diverging_hcl'
        fld.col <- 'Blue-Red'
        fld.col.inc <- 2
        fld.lev <- 't'
    } else {
        fld.name <- fld
        fld.range <- NA
        fld.palette.func <- 'diverging_hcl'
        fld.col <- 'Blue-Red'
        fld.col.inc <- NA
        fld.lev <- 't'
    }

    # Acquire packages
    source("/home/rmt001/R/my.filled.contour.R")
    require(colorspace)

    # Retrieve coordinate vectors
    coord.rad<-scan(paste(results.dir,run,'coord_rad.dat',sep='/'))
    coord.rad.r<-rev(coord.rad)
    coord.z <- list('m'=read.table(paste(results.dir,run,'coord_zm.dat',sep='/'))$V2,
                    't'=read.table(paste(results.dir,run,'coord_zt.dat',sep='/'))$V2)

    # Retrieve data
    d <- read.table(paste0(results.dir,'/',run,'/',fld,'/azmean_',fcst,'.dat'))
    if (max(d) == min(d)){ next }
    d <- d * fld.mult
    
    # Invert coordinate vectors if necessary
    if (coord.z$m[1] > coord.z$m[2]){
        coord.z$m <- rev(coord.z$m)
        coord.z$t <- rev(coord.z$t)
        d <- d[order(nrow(d):1),]
    }
    if (coord.rad[1] > coord.rad[2]){
        coord.rad <- rev(coord.rad)
        d <- d[, order(ncol(d):1)]
    }

    # Automatic configuration
    if (is.na(fld.range[1])){
        ext <- max(abs(d))
        fld.range <- c(-ext, ext)
        fld.col.inc <- signif(1.1*ext/10., 1)
    }

    # Plot 3D fields
    coord.z <- coord.z[[fld.lev]] / 1000
    levs <- seq(fld.range[1], fld.range[2], by=fld.col.inc)
    nlevs <- length(levs)
    postscript(paste(run,'_',fld,'_rh_',fcst,'.ps',sep=''), paper='special',
               width=7, height=7, bg='white', horiz=FALSE)    
    my.filled.contour(coord.rad.r, coord.z, t(d), zlim=fld.range, levels=levs,
                      xlim=c(0, radius), ylab='Height (km)', xlab='Radius (deg)',
                      ylim=c(0, height.max), main=paste(fld.name,' in ',run.name,' (',fcst,'h)',sep=''),
                      yaxt='n', col=rev(rainbow(nlevs,end=4/6)),
                      cex.axis=1.4, cex.lab=1.4,
                      key.title=title(main=expression('   '~m~s^{-1}), cex.main=1.2, line=1.5),
                      plot.axes={
                          axis(1, cex.axis=1.4);
                          axis(2, at=seq(0, height.max, by=2), cex.axis=1.4, las=2)},
                      key.axes={axis(4, cex.axis=1.2)})
    dev.off()
}
