# User configuration
runs<-c('dg25ctl','dg25offb51')
range<-240
wind.k<-8
results.dir<-'../results'
cols<-c('blue','red')

# Run-specific configuratoin
run.names <- c()
runs.std <- sub('TC_idealized_520a19_', '', runs) 
for (run in runs.std){
    if (run == 'dg25ctl'){
        run.names <- c(run.names, 'GEM Control')
    } else if (run == 'dg25offb51'){
        run.names <- c(run.names, 'OFFB51')
    } else {
        run.names <- c(run.names, run)
    }
}

# Field-specific configuration
fld.leg.pos <- 'bottomleft'

# Plot setup
postscript(paste(paste(runs,collapse='-'),'_windPres.ps',sep=''), paper='special',
           width=7, height=7, bg='white', horiz=FALSE)
default <- par(mar=c(5,5,4,2))
plot(1, 1, type='n', xlim=c(0,60), ylim=c(940,1010),
     xlab=expression('Maximum Wind Speed (m s'^'-1'*')'), ylab='',
     cex.lab=1.4, cex.main=1.4, cex.axis=1.4, las=1)
mtext('Central Pressure (hPa)', 2, line=3.5, cex=1.4)


# Plot data for all requested runs
col <- 0
for (run in runs){

    # Retrieve inputs
    d.wind <- c()
    d.pres <- c()
    for (i in seq(6, 240, by=6)){
        fcst <- sprintf("%03d", i)
        fname.wind <- paste0(results.dir,'/',run,'/VMAX/azmean_',fcst,'.dat')
        fname.pres <- paste0(results.dir,'/',run,'/P0/azmean_',fcst,'.dat')        
        if (file.exists(fname.wind) & file.exists(fname.pres)){
            d.wind<-rbind(d.wind, read.table(fname.wind)[wind.k])
            d.pres<-rbind(d.pres, read.table(fname.pres))
        }

    }
    series.wind <- d.wind[,1]
    series.pres <- apply(d.pres[, order(ncol(d.pres):1)], 1, min)
    
    # Generate plot
    col <- col+1
    points(series.wind, series.pres, col=cols[col], pch=19)
}
legend(fld.leg.pos, inset=0.01, legend=run.names, col=cols, cex=1.2, lwd=3)
par <- default
dev.off()
