# User configuration
run<-'wrfld01'
range<-192
radius<-3
results.dir<-'../results'

# Field-specific configuration
fld.mult <- 1
fld<-'RT'
fld.range <- c(60, 10)
fld.palette.func <- 'sequential_hcl'
fld.col <- 'GnBu'
fld.col.inc <- 2.5
fld.units <- expression(mm~h^-1)
fld.mult <- 1000 * 3600

# Acquire packages
source("/home/rmt001/R/my.filled.contour.R")
require(colorspace)

# Retrieve coordinate vectors
coord.rad<-scan(paste0(results.dir,'/',run,'/00/coord_rad.dat'))
coord.rad.r<-rev(coord.rad)

# Retrieve 2D fields
d.raw <- c()
d.wind.raw <- c()
coord.t <- c()
coord.t.wind <- c()
for (i in 0:range){
    fcst <- sprintf("%03d", i)
    fname <- paste0(results.dir,'/',run,'/00/',fld,'/azmean_',fcst,'.dat')
    if (file.exists(fname)){
        d.raw <- rbind(d.raw, read.table(fname))
        coord.t <- c(coord.t, i)
    }
    fname <- paste0(results.dir,'/',run,'/00/GZM/azmean_',fcst,'.dat')
    if (file.exists(fname)){
        hght <- read.table(fname)
        wind <- read.table(paste0(results.dir,'/',run,'/00/UU/azmean_',fcst,'.dat'))
        wind.lev <- c()
        for (rad in 1:ncol(hght)){
            wind.lev <- c(wind.lev, spline(hght[,rad], wind[,rad], xout=200.)$y)
        }
        d.wind.raw <- rbind(d.wind.raw, wind.lev)
        coord.t.wind <- c(coord.t.wind, i)
    }
}
d.r <- d.raw[, order(ncol(d.raw):1)] * fld.mult
d.wind <- d.wind.raw[, order(ncol(d.wind.raw):1)]

# Generate field for RMW
d.rmw <- 1.
col.rmw <- rep('black', times=nrow(d.wind))
for (i in 2:nrow(d.wind)){
    ind.max <- which(d.wind[i,]==max(d.wind[i,]))[1]
    new.rmw <- coord.rad.r[ind.max]
    if (new.rmw - d.rmw[length(d.rmw)] > 0.15){col.rmw[i-1] <- NA}
    d.rmw <- c(d.rmw, new.rmw)
}
#d.rmw.smooth <- loess(d.rmw~coord.t.wind, degree=1, span=1/(2*range/24), surface='direct')$fitted



levs <- seq(min(fld.range), max(fld.range), by=fld.col.inc)
nlevs <- length(levs)
postscript(paste(run,'_',fld,'_azhov.ps',sep=''), paper='special',
           width=7, height=7, bg='white', horiz=FALSE)
col.pal <- get(fld.palette.func)(nlevs, palette=fld.col)
if (fld.range[1] > fld.range[2]){col.pal <- rev(col.pal)}
my.filled.contour(coord.rad.r, coord.t, t(d.r), zlim=fld.range, levels=levs,
                  xlim=c(0, radius), ylab='Lead Time (h)', xlab='Radius (deg)',
                  yaxt='n', col=col.pal, cex.axis=1.4, cex.lab=1.4,
                  plot.axes={
                      .filled.contour(coord.rad.r, coord.t, t(d.r),
                                      levels=c(1.,min(fld.range)), col='lightgrey');
                      abline(h=144, col='darkgrey', lwd=2, lty=2);
                      segments(d.rmw[1:(length(d.rmw)-1)], coord.t.wind[1:(length(coord.t.wind)-1)], d.rmw[2:length(d.rmw)], coord.t.wind[2:length(coord.t.wind)], col=col.rmw);
                      axis(1, cex.axis=1.4);
                      axis(2, at=seq(min(coord.t), max(coord.t), by=24),
                           cex.axis=1.4, las=2)},
                  key.title={
                      mtext(expression('Rainfall Rate (mm h'^{-1}*')'), side=4, cex=1.4, line=3, las=0);
#                      mtext(fld.units, side=3, las=0, line=2.5, cex=1.2, adj=0, padj=1)
                  },
                  key.axes={axis(4, cex.axis=1.2)})
#contour(coord.rad.r, coord.t, t(d.rmw), levels=0., drawlabels=FALSE, add=TRUE)
dev.off()
