# User configuration
run<-'wrfld01'
interval<-1
fcst.in<-144
fcst<-sprintf("%03d", fcst.in)
radius<-3
height.max<-18
results.dir<-'../results'

# Loop over all fields to generate plots
fld.name <- 'Secondary Circulation'
fld.range <- c(-10, 10)
fld.palette.func <- 'diverging_hcl'
fld.col <- 'Blue-Red'
fld.col.inc <- 1

# Acquire packages
source("/home/rmt001/R/my.filled.contour.R")
require(colorspace)
require(fields)

# Retrieve coordinate vectors
coord.rad<-scan(paste(results.dir,run,'/00/coord_rad.dat',sep='/'))
coord.rad.r<-rev(coord.rad)
coord.z <- list('m'=read.table(paste(results.dir,run,'/00/coord_zm.dat',sep='/'))$V2/1000.,
                    't'=read.table(paste(results.dir,run,'/00/coord_zt.dat',sep='/'))$V2/1000.)

# Retrieve data
ww <- read.table(paste0(results.dir,'/',run,'/00/WW/azmean_',fcst,'.dat'))
vr <- read.table(paste0(results.dir,'/',run,'/00/VR/azmean_',fcst,'.dat'))
px <- read.table(paste0(results.dir,'/',run,'/00/PXT/azmean_',fcst,'.dat')) * 100
tt <- read.table(paste0(results.dir,'/',run,'/00/TT/azmean_',fcst,'.dat')) + 273.15
gzm <- read.table(paste0(results.dir,'/',run,'/00/GZM/azmean_',fcst,'.dat')) * 10
wz <- (-287. * tt) / (px * 9.81) * ww

# Invert coordinate vectors if necessary
if (coord.z$m[1] > coord.z$m[2]){
    coord.z$m <- rev(coord.z$m)
    coord.z$t <- rev(coord.z$t)
    wz <- wz[order(nrow(wz):1),]
    vr <- vr[order(nrow(vr):1),]
}
if (coord.rad[1] > coord.rad[2]){
    coord.rad <- rev(coord.rad)
    wz <- wz[, order(ncol(wz):1)]
    vr <- vr[, order(ncol(vr):1)]
}

# Create coordinate cubes for arrow plotting
coord.rad.r.2d <- c()
for (i in 1:length(coord.rad.r)){
    coord.rad.r.2d <- rbind(coord.rad.r.2d, rep(coord.rad.r[i], times=length(coord.z$m)))
}

# Plot 3D fields
levs <- seq(fld.range[1], fld.range[2], by=fld.col.inc)
nlevs <- length(levs)
postscript(paste(run,'_',fld,'_rh_',fcst,'.ps',sep=''), paper='special',
           width=7, height=7, bg='white', horiz=FALSE)
my.filled.contour(coord.rad, coord.z$m, t(vr), zlim=fld.range, levels=levs,
                  xlim=c(0, radius), ylab='Height (km)', xlab='Radius (deg)',
                  ylim=c(0, height.max),
                  yaxt='n', col=get(fld.palette.func)(nlevs, palette=fld.col),
                  cex.axis=1.4, cex.lab=1.4,
                  plot.axes={
#                      arrow.plot(as.vector(coord.rad.r.2d), as.vector(t(gzm))/1000., u=as.vector(t(-vr)), v=as.vector(t(wz)), xpd=FALSE, arrow.ex=0.05, length=0.05);
                      axis(1, cex.axis=1.4);
                      axis(2, at=seq(0, height.max, by=2), cex.axis=1.4, las=2)},
                  key.axes={axis(4, cex.axis=1.2)})
dev.off()
