# User configuration
run<-'i5ectl1'
interval<-1
fcst.in<-144
fcst<-sprintf("%03d", fcst.in)
radius<-3
height.max<-18
results.dir<-'../results'

# Loop over all fields to generate plots
fld.range <- c(-40, 40)
fld.palette.func <- 'diverging_hcl'
fld.col <- 'Blue-Red'
fld.col.inc <- 5

# Acquire packages
source("/home/rmt001/R/my.filled.contour.R")
require(colorspace)
require(fields)
require(akima)

# Retrieve coordinate vectors
coord.rad<-scan(paste(results.dir,run,'/00/coord_rad.dat',sep='/'))
coord.rad.r<-rev(coord.rad)
coord.z <- list('m'=read.table(paste(results.dir,run,'/00/coord_zm.dat',sep='/'))$V2,
                't'=read.table(paste(results.dir,run,'/00/coord_zt.dat',sep='/'))$V2)

# Retrieve data
ww <- read.table(paste0(results.dir,'/',run,'/00/WW/azmean_',fcst,'.dat'))
vr.d <- read.table(paste0(results.dir,'/',run,'/00/VR/azmean_',fcst,'.dat'))
px <- read.table(paste0(results.dir,'/',run,'/00/PXT/azmean_',fcst,'.dat')) * 100.
tt <- read.table(paste0(results.dir,'/',run,'/00/TT/azmean_',fcst,'.dat')) + 273.15
gzm.d <- read.table(paste0(results.dir,'/',run,'/00/GZM/azmean_',fcst,'.dat')) * 10
wz.d <- (-287. * tt) / (px * 9.81) * ww

# Invert coordinate vectors if necessary
invert <- function(d, dir){
    if (dir==1){
        return(d[order(nrow(d):1),])
    } else if (dir==2){
        return(d[, order(ncol(d):1)])
    } else {
        stop('unknown direction')
    }
}
if (coord.z$m[1] > coord.z$m[2]){
    coord.z$m <- rev(coord.z$m)
    coord.z$t <- rev(coord.z$t)
    vr <- invert(vr.d, 1)
    wz <- invert(wz.d, 1)
    gzm <- invert(gzm.d, 1)
}
if (coord.rad[1] > coord.rad[2]){
    coord.rad <- rev(coord.rad)
    vr <- invert(vr.d, 2)
    wz <- invert(wz.d, 2)
    gzm <- invert(gzm.d, 2)
}
coordz <- coord.z$m / 1000

# Automatic configuration
if (is.na(fld.range[1])){
    ext <- max(abs(d))
    fld.range <- c(-ext, ext)
    fld.col.inc <- signif(1.1*ext/10., 1)
}

# Create coordinate table for vectors
coord.rad.r.2d <- c()
for (i in 1:length(coord.rad.r)){
    coord.rad.r.2d <- rbind(coord.rad.r.2d, rep(coord.rad.r[i], times=length(coord.z$m)))
}

# Prepare vectors for plotting
hght.v <- seq(0.5, height.max, by=1)
rad.v <- seq(0, radius, by=0.1)
vr.v <- interp(coord.rad.r.2d, t(gzm)/1000., t(vr), xo=rad.v, yo=hght.v, extrap=TRUE)
wz.v <- interp(coord.rad.r.2d, t(gzm)/1000., t(wz), xo=rad.v, yo=hght.v, extrap=TRUE)

# Convert to plot units and thin for small values
cvt.rad <- 1/111000.
cvt.hght <- 1/1000.
spd.min <- 0.00035
spd.v <- sqrt((vr.v$z * cvt.rad * height.max/radius)^2 + (wz.v$z * cvt.hght)^2)
x.v.plot <- radius+1
y.v.plot <- 0
vr.v.plot <- 1
wz.v.plot <- 0
for (i.hght in 1:ncol(vr.v$z)){
    for (i.rad in 1:nrow(vr.v$z)){
        if (spd.v[i.rad, i.hght] > spd.min){
            x.v.plot <- c(x.v.plot, rad.v[i.rad])
            y.v.plot <- c(y.v.plot, hght.v[i.hght])
            vr.v.plot <- c(vr.v.plot, vr.v$z[i.rad, i.hght] * cvt.rad)
            wz.v.plot <- c(wz.v.plot, wz.v$z[i.rad, i.hght] * cvt.hght)
        }
    }
}

# Reference vector
x.v.plot <- c(x.v.plot, rep(1.9, times=2))
y.v.plot <- c(y.v.plot, rep(6, times=2))
vr.v.plot <- c(vr.v.plot, c(20*cvt.rad, 0))
wz.v.plot <- c(wz.v.plot, c(0, 2*cvt.hght))

# Plot 3D fields
levs <- seq(fld.range[1], fld.range[2], by=fld.col.inc)
nlevs <- length(levs)
postscript(paste(run,'_sec_rh_',fcst,'.ps',sep=''), paper='special',
           width=7, height=7, bg='white', horiz=FALSE)
my.filled.contour(coord.rad.r, coordz, t(vr), zlim=fld.range, levels=levs,
                  xlim=c(0, radius), ylab='Height (km)', xlab='Radius (deg)',
                  ylim=c(0, height.max),
                  yaxt='n', col=get(fld.palette.func)(nlevs, palette=fld.col),
                  cex.axis=1.4, cex.lab=1.4,
                  plot.axes={
#                      contour(coord.rad.r, coordz, t(tphy), add=TRUE);
                      polygon(c(1.8,1.8,2.9,2.9,1.8), c(5,7.5,7.5,5,5), col='white');
                      text(2.05, 6.65, expression('2 m s'^{-1}*' updraft'), pos=4);
                      text(2, 5.85, expression('20 m s'^{-1}*' outflow'), pos=4);
                      arrow.plot(x.v.plot, y.v.plot, u=vr.v.plot, v=wz.v.plot, xpd=FALSE, arrow.ex=140, length=0.075);
                      axis(1, cex.axis=1.4);
                      axis(2, at=seq(0, height.max, by=2), cex.axis=1.4, las=2)},
                  key.title={
                      mtext(expression('Radial Wind Speed (m s'^{-1}*')'), side=4, cex=1.4, line=3, las=0);
#                      mtext(expression('m s'^{-1}), side=3, las=0, line=2.5, cex=1.2, adj=0, padj=1)
                  },
                  key.axes={axis(4, cex.axis=1.2)})
dev.off()

