# User configuration
runs<-c('wrflsdd01','i5lnfspindown2','i5lonfspindown2','i5lOnfspindown2')
runs.names<-c('WRF Reference', 'GEM Control', 'OFFB5', 'OFFB51')
runs.col<-c('black', 'blue', 'magenta', 'red')
range<-120
fld<-'P0'
x.offset<-list('wrflsdd01'=-144)
results.dir<-'../results'

# Field-specific configuration
fld.mult <- 1
fld.min <- -Inf
fld.leg.pos <- 'bottomleft'
if (fld == 'P0'){
    fld.name <- 'Minimum Central Pressure'
    fld.range <- c(890,1008)
    fld.leg.pos <- 'bottomright'
    if (sum(grep('late', runs)) > 0){fld.leg.pos <-'bottomright'}
    fld.units <- bquote(.(fld.name) ~ "(hPa)")
    fld.type <- 'min'
} else if (fld == 'V10'){
    fld.name <- 'Maximum Wind'
    fld.range <- c(0,35)
    fld.units <- bquote(.(fld.name) ~ "(" * m~s^-1 * ")")
    fld.type <- 'max'
    fld.leg.pos <- 'topleft'
    fld.min <- 0
} else if (fld == 'V10R'){
    fld.name <- 'Maximum Inflow Wind'
    fld.range <- c(-20,20)
    fld.units <- bquote(.(fld.name) ~ "(" * m~s^-1 * ")")
    fld.type <- 'min'
} else if (fld == 'V10T'){
    fld.name <- 'Maximum Tangential Wind'
    fld.range <- c(0,100)
    fld.units <- bquote(.(fld.name) ~ "(" * m~s^-1 * ")")
    fld.type <- 'max'
    fld.leg.pos <- 'topleft'
    fld.min <- 0
} else if (fld == 'RT'){
    fld.name <- 'Precipitation Rate'
    fld.mult <- 1000*3600
    fld.range <- c(0,200)
    fld.units <- bquote(.(fld.name) ~ "(" * mm~h^-1 * ")")
    fld.type <- 'max'
} else {
    stop(paste("Error: Unknown field fld =",fld))
}

# Retrieve coordinate vectors
coord.rad<-scan(paste0(results.dir,'/',runs[1],'/coord_rad.dat'))
coord.rad.r<-rev(coord.rad)

# Plot setup
postscript(paste(paste(runs,collapse='-'),'_',fld,'_series.ps',sep=''), paper='special',
           width=8, height=6, bg='white', horiz=FALSE)
default <- par(mar=c(5,6,4,2))
plot(1, 1, type='n', xlim=c(0,range), ylim=fld.range, xaxt='n', yaxt='n',
     xlab='Lead Time (h)', ylab='', cex.lab=1.4, xaxs='i',
     main=fld.name, cex.main=1.4)
mtext(fld.units, 2, line=4, cex=1.4)
axis(1, at=seq(0, range, by=24),cex.axis=1.4)
axis(2, at=pretty(fld.range), cex.axis=1.4, las=2);

# Plot data for all requested runs
col <- 0
for (j in 1:length(runs)){
    run <- runs[j]

    # Retrieve inputs
    d.raw <- c()
    coord.t <- c()
    for (i in 0:240){
        fcst <- sprintf("%03d", i)
        fname <- paste0(results.dir,'/',run,'/',fld,'/azmean_',fcst,'.dat')
        if (file.exists(fname)){
            d.raw<-rbind(d.raw,read.table(fname))
            coord.t <- c(coord.t, i)
        }
    }
    d.r<-d.raw[, order(ncol(d.raw):1)] * fld.mult
    if (fld.type == 'min'){
        series <- apply(d.r, 1, min)
    } else if (fld.type == 'max'){
        series <- apply(d.r, 1, max)
    } else if (fld.type == 'mean'){
        series <- apply(d.r, 1, mean)
    } else {
        stop(paste("Unknown field type:",fld.type))
    }

    # Generate plot
    col <- col+1
    offset <- x.offset[[run]]
    if (is.null(offset)) offset <- 0
    valid <- series > fld.min
    lines(offset + coord.t[valid], series[valid], col=runs.col[j], lwd=3)
}
legend(fld.leg.pos, inset=0.01, legend=runs.names, col=runs.col, cex=1.2, lwd=3)
par <- default
dev.off()
