program process

  !  Associate required modules - MODIFY to include additional modules
  use Std_Variables, only: &
       ni,nj,nk,ntimes,t,progType,fileType,labl,dx,dy,dgrw,dim                  !variables
  use IO_Params, only: &
       input,output,gridFile,infoFld, &                                         !variables
       getParams                                                                !subprograms
  use Dimensions_Constructor, only: &
       dxs,dys, &                                                               !variables
       makeDim                                                                  !subprograms
  use TopLevel, only: &
       inOpen,outOpen,timer,fileTimer,projParam,lat,long,lev,vCoord,mapProj,gotLat,&!variables
       openGridFiles,closeGridFiles,getGridSize,getGridInfo,timeLoop            !subprograms
  use FileIO, only: &
       readGrid,writeGrid                                                       !subprograms
  use FileInfo_Type
  use Dimensions_Type
  use FST_File_Utilities
  use String
  use Math
  use NCAR_Utilities
  use Constants
  use Dynamics
  use NCAR_Utilities
  use ifport, only: makedirqq
  use Thermodynamics
! use Interface_MRG

! load FST       <- force comp90.d loading of ARMNLIB (FST library)
! load GEMPAK    <- force comp90.d loading of GEMLIB (GEMPAK library)
! load CDF       <- force comp90.d loading of NETCDF (netCDF library)

  !  This program template serves as the basis for the developer's
  !  interface for the MRG Interactive MetCal program and library.  
  !  Simple setup and IO operations are performed here; the developer
  !  is expected to extend upon this code as needed during further
  !  subprogram development for the library.

  implicit none

  !  Program Name: 
  !  Version: 1.0
  !  Programmer: 
  !  Date Modified:
  !  Modified by:

!!$    MRG_Interactive MetCal metorological calculator - developer's template
!!$    Copyright 2001 (C) Ron McTaggart-Cowan
!!$
!!$    This file is part of MRG Interactive MetCal (MIM).
!!$
!!$    MIM is free software; you can redistribute it and/or modify
!!$    it under the terms of the GNU General Public License as published by
!!$    the Free Software Foundation; either version 2 of the License or
!!$    any later version.
!!$
!!$    MIM is distributed in the hope that it will be useful,
!!$    but WITHOUT ANY WARRANTY; without even the implied warranty of
!!$    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!$    GNU General Public License for more details.
!!$
!!$    You should have received a copy of the GNU General Public License
!!$    along with MIM; if not, write to the Free Software
!!$    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
!!$    MA  02111-1307  USA

  !  Derived types
  type vectorfield
     character(len=4) :: u,v    !Cartesian wind components
     character(len=4) :: t,r    !Cylindrical wind components
     real :: mult               !Multiplication factor (unit conversion)
  end type vectorfield
  
  !  User-defined parameters
  integer, parameter :: MAXHR=240
  real, parameter :: AZRAD=5.                   !Radius of azimuthal-mean calculations (deg)
  real, parameter :: AZRES=0.05                  !Radial resolution for az-mean calculations (deg)
  real, parameter :: MAXRAD=1000000.            !Maximum TC radius in meters
  character(len=4), dimension(5) :: &
       AZ_SFC_FIELDS=(/'P0  ','RT  ','FC:5','FV:5','FQ  '/) !Surface fields for azimuthal means
  character(len=4), dimension(8) :: &
       AZ_3D_SCALAR_FIELDS=(/'WW','KM','KT','HU','HR','TPHY','TI','TT'/)
  type(vectorfield), dimension(4) :: AZ_3D_VECTOR_FIELDS=(/ &
       vectorfield('UU','VV','VT','VR', kttoms), &
       vectorfield('UPHY','VPHY','VTP','VRP', 1.), &
       vectorfield('UADV','VADV','VTD','VRD', 1.), &
       vectorfield('TU', 'TV', 'VTBL', 'VRBL', 1.) &
       /)
  
  !  Declaration of variables - MODIFY to declare additional variables
  integer :: i,j,k,fcst,tci,tcj,fdout,sfclev,prog,di,dj,li,lj,hw,ii,jj,ltci,ltcj, &
       dcmip_bot_levs,fldi
  integer, dimension(2) :: pos
  real :: circ,circmax
  real, dimension(0:MAXHR) :: pres,maxwind10,tclat,tclon
  real, dimension(:), allocatable :: gzm,gzt,maxspd
  real, dimension(:), pointer :: am1d=>null(),am1dr=>null(),am1dt=>null(), &
       rcoord=>null(),levm=>null()
  real, dimension(:,:), allocatable :: spd10
  real, dimension(:,:), pointer :: u10=>null(),v10=>null(),am2d=>null(), &
       f2d=>null(),am2dt=>null(),am2dr=>null(),p0=>null()
  real, dimension(:,:,:), allocatable :: spd
  real, dimension(:,:,:), pointer :: gz=>null(),uu=>null(),vv=>null(), &
       f3d=>null(),tt=>null(),hu=>null(),px=>null(),hr=>null()
  character(len=1) :: clev
  character(len=3) :: cfcst,cprog
  character(len=4) :: sfcfld
  character(len=1024) :: fbase,obase,cname
  logical :: is_dcmip=.false.
  type(dimensions) :: dimm

  !  Retrieve command line arguments
  i = command_argument_count()
  if (i < 1) then
     write(0,*) 'USAGE: vitals HOUR'
     stop
  endif
  call get_command_argument(1, cprog)
  read(cprog,'(i)') prog
  
  !  Get user-defined parameters                                        (use IO_Params)
  call getParams(input,output,gridFile,fileType)
  fbase = input(1)%fileName
  if (index(input(2)%fileName, 'results/d') > 0) is_dcmip = .true.
  input(1)%fileName = trim(fbase)//cprog
  obase = output(1)%fileName
  fdout = output(1)%unit

  !  Initialization and opening                                         (use TopLevel)
  allocate(inOpen(size(input)))
  allocate(outOpen(size(output)))
  inOpen = .true. ; inOpen(2:) = .false.; outOpen = .false.
  call openGridFiles(inOpen,outOpen,fileType)
  do i=1,size(input)
     if (.not.inOpen(i)) open(input(i)%unit, file=input(i)%fileName)
  enddo

  !  Get grid dimensions from primary input file                        (use TopLevel)
  infoFld = 'P0'
  if (field(input(1), 'TT', quiet=.true.)) infoFld = 'TT'
  call getGridSize(input(1),fileType,ni,nj,nk,ntimes,                   &
       mapFile=gridFile,infoFld=infoFld)

  !  Set space
  allocate(gzm(nk), gzt(nk))
  allocate(spd10(ni,nj))
  allocate(spd(ni,nj,nk))
  allocate(maxspd(nk))
  
  !  Get file information (take lat/long from grid file)                (use TopLevel)
  call getGridInfo(input(1),fileType,ni,nj,nk,ntimes,lev,               &
             vCoord,fileTimer,labl,progType,dx,dy,mapProj,dgrw,infoFld, &
             gotLat,lat,long,gridFile,projParam)
  
  !  Construct dim                                                      (use Dimensions_Constructor)
  if (gotLat) then
     call makeDim(ni,nj,nk,dx,dy,lat,long,dxs,dys,lev,vCoord,infoFld,   &
	  fileType,progType,dim,mapProj=mapProj,projParam=projParam)
  else
     call makeDim(ni,nj,nk,dx,dy,lev,vCoord,infoFld,fileType,progType,	&
	  dim)
  endif

  !  Create momentum grid specification
  infoFld = 'P0'
  if (field(input(1), 'UU', quiet=.true.)) infoFld = 'UU'
  call getGridInfo(input(1),fileType,ni,nj,nk,ntimes,levm,              &
             vCoord,fileTimer,labl,progType,dx,dy,mapProj,dgrw,infoFld, &
             gotLat,lat,long,gridFile,projParam)
  call makeDim(ni,nj,nk,dx,dy,lat,long,dxs,dys,levm,vCoord,infoFld,   &
	  fileType,progType,dimm,mapProj=mapProj,projParam=projParam)

  !  Create approximate vertical coordinate  
  call readGrid(input(1), 'GZ', (/-1,-1,-1/), dim, gz)
  do k=1,dim%nk
     gzt(k) = sum(gz(:,:,k))/real(dim%ni*dim%nj) * damtom
  enddo
  call readGrid(input(1), 'GZ', (/-1,-1,-1/), dimm, gz)
  do k=1,dim%nk
     gzm(k) = sum(gz(:,:,k))/real(dim%ni*dim%nj) * damtom
  enddo
  deallocate(gz)

  !  Process each line of the track file
  pres(:) = 0.
  do
     read(input(2)%unit, *, end=999) fcst, tclat(fcst), tclon(fcst), &
          tci, tcj, pres(fcst)
     if (fcst /= prog) cycle
     call closeFST(input(1))     
     write(cfcst, '(i3)') fcst
     cfcst = lpad(cfcst, '0')
     input(1)%fileName = trim(fbase)//cfcst
     call openFST(input(1), type='input')

     ! Maximum wind
     call readGrid(input(1), 'P0', 0., (/-1,-1,-1/), dim, p0)
     pos = minloc(p0)
     if (abs(pos(1)-tci) > 10 .or. abs(pos(2)-tcj) > 10) then
        tci = pos(1)
        tcj = pos(2)
        tclat(fcst) = dim%lat(tci, tcj)
        tclon(fcst) = dim%long(tci, tcj)
     endif
     if (field(input(1), 'UD', quiet=.true.)) then
        call readGrid(input(1), 'UD', 0., (/-1,-1,-1/), dim, u10)
        call readGrid(input(1), 'VD', 0., (/-1,-1,-1/), dim, v10)
     else
        call readGrid(input(1), 'UU', dimm%levs(1), (/-1,-1,-1/), dim, u10); u10 = u10*KTTOMS
        call readGrid(input(1), 'VV', dimm%levs(1), (/-1,-1,-1/), dim, v10); v10 = v10*KTTOMS
     endif
     spd10 = windspd(u10, v10, dim)
     pos = maxloc(spd10)
     if (pathLength(pos(1), pos(2), tci, tcj, dim) > MAXRAD) then
        print*, 'stats: ', pos(1),pos(2),tci, tcj, pathLength(pos(1), pos(2), tci, tcj, dim),MAXRAD
        write(0,*) 'Maximum wind too far from cyclone centre at ',pos
        maxwind10(fcst) = -1.
     else
        maxwind10(fcst) = spd10(pos(1), pos(2))
     endif
     i = tci
     do while (pathLength(i, tcj, tci, tcj, dim) < MAXRAD .and. i < dim%ni)
        i = i+1        
     enddo
     di = i-tci
     j = tcj
     do while (pathLength(tci, j, tci, tcj, dim) < MAXRAD .and. j < dim%nj)
        j = j+1
     enddo
     dj = j-tcj
     
     ! Compute azimuthal-mean for surface fields
     do fldi=1,size(AZ_SFC_FIELDS)
        sfcfld = AZ_SFC_FIELDS(fldi)
        if (index(sfcfld, ':') > 0) then
           clev = sfcfld(index(sfcfld, ':')+1:)
           sfcfld = sfcfld(1:index(sfcfld, ':')-1)
           read(clev,'(i)') sfclev
           cname = 'NONE'
        else
           sfclev = 0
           cname = dim%vCoord
        endif
        if (field(input(1), sfcfld, quiet=.true.)) then
           call readGrid(input(1), sfcfld, real(sfclev), (/-1,-1,-1/), dim, f2d, &
                verticalCoordinateName=cname)
           call azimuthalMean(f2d, dim, am1d, centre=(/tclat(fcst), tclon(fcst)/), &
                resolution=AZRES, radius=AZRAD, coord=rcoord)
           call write1d(am1d, sfcfld)        
        endif
     enddo
     call azimuthalMean(u10, v10, dim, am1dt, am1dr, &
          centre=(/tclat(fcst), tclon(fcst)/), resolution=AZRES, radius=AZRAD)
     call write1d(am1dt, 'V10T')
     call write1d(am1dr, 'V10R')
     call azimuthalMean(spd10, dim, am1d, centre=(/tclat(fcst), tclon(fcst)/), &
          resolution=AZRES, radius=AZRAD, coord=rcoord)
     call write1d(am1d, 'V10')
     
     ! Compute azimuthal-mean section for 3D fields
     if (mod(fcst, 6) == 0) then
        ! Vector fields
        do fldi=1,size(AZ_3D_VECTOR_FIELDS)
           call readGrid(input(1), AZ_3D_VECTOR_FIELDS(fldi)%u, (/-1,-1,-1/), dimm, uu)
           uu = uu * AZ_3D_VECTOR_FIELDS(fldi)%mult
           call readGrid(input(1), AZ_3D_VECTOR_FIELDS(fldi)%v, (/-1,-1,-1/), dimm, vv)
           vv = vv * AZ_3D_VECTOR_FIELDS(fldi)%mult
           call azimuthalMean(uu, vv, dim, am2dt, am2dr, &
                centre=(/tclat(fcst), tclon(fcst)/), resolution=AZRES, radius=AZRAD)
           call write2d(am2dt, AZ_3D_VECTOR_FIELDS(fldi)%t)
           call write2d(am2dr, AZ_3D_VECTOR_FIELDS(fldi)%r)
           spd = windspd(uu, vv, dim)
           call azimuthalMean(spd, dim, am2d, &
                centre=(/tclat(fcst), tclon(fcst)/), resolution=AZRES, radius=AZRAD)
           call write2d(am2d, AZ_3D_VECTOR_FIELDS(fldi)%u)
           if (AZ_3D_VECTOR_FIELDS(fldi)%u == 'UU') then
              do k=1,dim%nk
                 maxspd(k) = maxval(spd(max(tci-di,1):min(tci+di,dim%ni), max(tcj-dj,1):min(tcj+dj,dim%nj), k))
              enddo
              call write1d(maxspd, 'VMAX')
              DCMIP_CENTERING: if (is_dcmip) then
                 hw = 2; dcmip_bot_levs = 10
                 nullify(am1d, am2d)
                 do k=1,min(dcmip_bot_levs, dim%nk)
                    circmax = -9999.
                    ltci = tci; ltcj = tcj
                    do j=tcj-hw,tcj+hw
                       do i=tci-hw,tci+hw
                          circ = 0.
                          ! south box
                          do jj=j-2*hw,j-hw
                             do ii=i-hw,i+hw
                                circ = circ + uu(ii,jj,k)
                             enddo
                          enddo
                          ! north box
                          do jj=j+hw,j+2*hw
                             do ii=i-hw,i+hw
                                circ = circ - uu(ii,jj,k)
                             enddo
                          enddo
                          ! west box
                          do jj=j-hw,j+hw
                             do ii=i-2*hw,i-hw
                                circ = circ - vv(ii,jj,k)
                             enddo
                          enddo
                          ! east box
                          do jj=j-hw,j+hw
                             do ii=i+hw,i+2*hw
                                circ = circ + vv(ii,jj,k)
                             enddo
                          enddo
                          if (circ > circmax) then
                             circmax = circ
                             ltci = i
                             ltcj = j
                          endif
                       enddo
                    enddo
                    call azimuthalMean(spd(:,:,k), dim, am1d, &
                         centre=(/dim%lat(ltci,ltcj), dim%long(ltci,ltcj)/), resolution=AZRES, radius=AZRAD)
                    if (.not.associated(am2d)) allocate(am2d(size(am1d), dcmip_bot_levs))
                    am2d(:, k) = am1d
                 enddo
                 call write2d(am2d, 'VMLEV')
              endif DCMIP_CENTERING
           endif
           deallocate(uu, vv)
        enddo
        ! Generic atmospheric fields
        do i=1,size(AZ_3D_SCALAR_FIELDS)
           if (AZ_3D_SCALAR_FIELDS(i) == 'HR' .and. .not.field(input(1), 'HR', quiet=.true.)) then
              if (.not.associated(hr)) allocate(hr(dim%ni, dim%nj, dim%nk))
              call readGrid(input(1), 'TT', (/-1,-1,-1/), dim, tt)
              call readGrid(input(1), 'HU', (/-1,-1,-1/), dim, hu)
              call readGrid(input(1), 'PX', (/-1,-1,-1/), dim, px)
              do k=1,dim%nk
                 hr(:,:,k) = hutohr(hu(:,:,k), tt(:,:,k), px(:,:,k))
              enddo
              deallocate(tt, hu, px)
              f3d => hr
           else
              call readGrid(input(1), AZ_3D_SCALAR_FIELDS(i), (/-1,-1,-1/), dim, f3d)
           endif
           call azimuthalMean(f3d, dim, am2d, &
                centre=(/tclat(fcst), tclon(fcst)/), resolution=AZRES, radius=AZRAD)
           call write2d(am2d, AZ_3D_SCALAR_FIELDS(i))
           deallocate(f3d)
        enddo
        ! Add momentum-level heights
        nullify(gz)
        call readGrid(input(1), 'GZ', (/-1,-1,-1/), dimm, gz)
        call azimuthalMean(gz, dimm, am2d, &
             centre=(/tclat(fcst), tclon(fcst)/), resolution=AZRES, radius=AZRAD)
        call write2d(am2d, 'GZM')
        nullify(gz)
        ! Add thermo-level pressures
        nullify(px)
        call readGrid(input(1), 'PX', (/-1,-1,-1/), dim, px)
        call azimuthalMean(px, dim, am2d, &
             centre=(/tclat(fcst), tclon(fcst)/), resolution=AZRES, radius=AZRAD)
        call write2d(am2d, 'PXT')
        nullify(px)
     endif
     
  enddo
999 continue

  ! Generate coordinate outputs
  open(fdout, file=trim(obase)//'coord_rad.dat')
  write(fdout, '(999(f,x))') (rcoord(i),i=1,size(rcoord))
  close(fdout)
  open(fdout, file=trim(obase)//'coord_zt.dat')
  do k=1,dim%nk
     write(fdout,'(2(f,x))') dim%levs(k), gzt(k)
  enddo
  close(fdout)
  open(fdout, file=trim(obase)//'coord_zm.dat')
  do k=1,dim%nk
     write(fdout,'(2(f,x))') dim%levs(k), gzm(k)
  enddo
  close(fdout)
  
  ! Generate basic outputs
  open(fdout, file=trim(obase)//'basic.dat')
  if (prog == 0) write(fdout,'(a)') 'fcst lat lon pres v10m'
  write(fdout,'(i,999(x,e))') prog, tclat(prog), tclon(prog), &
       pres(prog), maxwind10(prog)
  close(fdout)
  
  !  Close input/output files                                           (use TopLevel)
  call closeGridFiles(inOpen,outOpen,fileType)
  do i=1,size(input)
     if (.not.inOpen(i)) close(input(i)%unit)
  enddo

contains

  subroutine write1d(fld, name)
    real, dimension(:), intent(in) :: fld
    character(len=*), intent(in) :: name
    character(len=1024) :: odir
    integer :: i
    odir = trim(obase)//name
    call mkdir(odir)
    open(fdout, file=trim(odir)//'/azmean_'//cfcst//'.dat')
    write(fdout, '(999(x,e))') (fld(i),i=1,size(fld))
    close(fdout)
  end subroutine write1d

  subroutine write2d(fld, name)
    real, dimension(:,:), intent(in) :: fld
    character(len=*), intent(in) :: name
    character(len=1024) :: odir
    integer :: i,j
    odir = trim(obase)//name
    call mkdir(odir)
    open(fdout, file=trim(odir)//'/azmean_'//cfcst//'.dat')
    do j=1,size(fld, dim=2)
       write(fdout, '(999(x,e))') (fld(i,j),i=1,size(fld, dim=1))
    enddo
    close(fdout)
  end subroutine write2d

  subroutine mkdir(dir)
    character(len=*), intent(in) :: dir
    logical :: dir_exists
    inquire(directory=dir, exist=dir_exists)
    if (.not.dir_exists) then
       if (.not.makedirqq(dir)) then
          write(0,*) 'Cannot create '//trim(dir)
          return
       endif
    endif
  end subroutine mkdir

  !  End of main program
end program process
