# Configuration
plot.range <- c(880,1020)
by.region <- FALSE
year.name <- 2021
runs <- c('CMC', 'ECM', 'NCP', 'JMA')
runs.long <- c('CMC', 'ECMWF', 'NCEP', 'JMA')
runs.col <- c('blue', 'black', 'black', 'black')
runs.res <- c('0.135', '0.08', '0.117', '0.18')
runs.grid <- c('1', '0.13', '0.5', '0.25')

# Retrieve data
tc <- read.csv('WGNETC_DataFrame.csv', header=TRUE)
valid <- tc$best_pc > 0  #& tc$region == "ENP"

# Internal configurations
if (year.name == 2020){
    year <- tc$tcnum > 1999 & tc$tcnum < 2100  #2020
} else if (year.name == 2021){
    year <- tc$tcnum > 2099
} else {
    year <- tc$tcnum > 1999
}

# Plotting
postscript("wgne_gdps_2021.ps", bg="white", paper="special",
           horiz=FALSE, width=7, height=7)
default <- par(mar=c(5,7,4,2))
layout(matrix(seq(1,length(runs)), ncol=2, byrow=TRUE))
for (i in 1:length(runs)){
    centre <- tc$center == runs[i]
    plot(tc$best_pc[valid&year&centre], tc$fcst_pc[valid&year&centre],
         main=paste0("WGNE Evaluation for ",runs.long[i]," (", year.name, ")"),
         xlab="Best Track Central Pressure (hPa)", ylab=NA,
         cex.lab=1., cex.axis=1., cex.main=1., las=1, pch=19, cex=0.5,
         xlim=plot.range,ylim=plot.range, col=runs.col[i],
         panel.first={abline(0,1,col='grey',lty=2,lwd=3);})
    mtext(paste0(runs.long[i]," Central Pressure (hPa)"), 2, line=4, cex=1.)
    text(945, 905, bquote('Model: '~.(runs.res[i])^o), pos=4)
    text(945, 890, bquote('Data: '~.(runs.grid[i])^o), pos=4)
    text(875, 1015, runs.long[i], pos=4)
    if (by.region){
        regions <- levels(as.factor(tc$region))
        cnt <- 1
        for (r in regions){
            region <- tc$region==r
            points(tc$best_pc[valid&year&region], tc$fcst_pc[valid&year&region],
                   pch=19,col=cnt)
            cnt <- cnt+1
        }
        legend('bottomright', inset=0.05, legend=regions, col=seq(1,length(regions)),
               pch=19)
    }
}
par(default)
dev.off()
